import filecmp
import inspect
import logging
import os
import pathlib
from dataclasses import dataclass
from itertools import chain
from typing import Any, NamedTuple

import pytest
import yaml
from pytest_mock import MockerFixture
from pytest_subtests import SubTests

from competitive_verifier.documents.config import ConfigIcons, ConfigYaml
from competitive_verifier.documents.front_matter import split_front_matter_raw
from competitive_verifier.documents.main import main
from competitive_verifier.oj import check_gnu_time

from .data.user_defined_and_python import UserDefinedAndPythonData
from .types import FilePaths
from .utils import dummy_commit_time


@dataclass
class MarkdownData:
    path: str
    front_matter: dict[str, Any]
    content: bytes = b""


@dataclass
class DocsData:
    root: pathlib.Path
    dest_root: pathlib.Path
    targets_data: list[MarkdownData]
    default_args: list[str]
    user_defined_and_python_data: UserDefinedAndPythonData


@pytest.fixture
def data(
    file_paths: FilePaths,
    user_defined_and_python_data: UserDefinedAndPythonData,
) -> DocsData:
    conf = user_defined_and_python_data.config_dir_path
    verify = conf / "verify.json"
    result = conf / "result.json"
    return DocsData(
        root=file_paths.root,
        dest_root=file_paths.dest_root,
        user_defined_and_python_data=user_defined_and_python_data,
        default_args=[
            "--verify-json",
            str(verify),
            str(result),
        ],
        targets_data=[
            MarkdownData(
                path="awk/aplusb.awk.md",
                front_matter={
                    "data": {
                        "attributes": {"TITLE": 'Calculate "A + B"'},
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [],
                        "embedded": [
                            {
                                "code": pathlib.Path("awk/aplusb.awk").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "awk/aplusb.awk",
                        "pathExtension": "awk",
                        "requiredBy": [],
                        "timestamp": "1985-12-01 00:29:29.550000-07:00",
                        "verificationStatus": "LIBRARY_NO_TESTS",
                        "verifiedWith": [],
                        "title": 'Calculate "A + B"',
                    },
                    "documentation_of": "awk/aplusb.awk",
                    "layout": "document",
                    "title": 'Calculate "A + B"',
                },
            ),
            MarkdownData(
                path="awk/aplusb.test.awk.md",
                front_matter={
                    "data": {
                        "attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb"
                        },
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [],
                        "embedded": [
                            {
                                "code": pathlib.Path("awk/aplusb.test.awk").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": True,
                        "isVerificationFile": True,
                        "path": "awk/aplusb.test.awk",
                        "pathExtension": "awk",
                        "requiredBy": [],
                        "testcases": [
                            {
                                "elapsed": 3.96,
                                "environment": "awk",
                                "memory": 10.63,
                                "name": "example_00",
                                "status": "RE",
                            },
                            {
                                "elapsed": 4.87,
                                "environment": "awk",
                                "memory": 57.66,
                                "name": "example_01",
                                "status": "RE",
                            },
                            {
                                "elapsed": 9.41,
                                "environment": "awk",
                                "memory": 55.5,
                                "name": "random_00",
                                "status": "RE",
                            },
                            {
                                "elapsed": 8.94,
                                "environment": "awk",
                                "memory": 5.09,
                                "name": "random_01",
                                "status": "RE",
                            },
                            {
                                "elapsed": 6.02,
                                "environment": "awk",
                                "memory": 50.45,
                                "name": "random_02",
                                "status": "RE",
                            },
                            {
                                "elapsed": 0.29,
                                "environment": "awk",
                                "memory": 48.46,
                                "name": "random_03",
                                "status": "RE",
                            },
                            {
                                "elapsed": 7.92,
                                "environment": "awk",
                                "memory": 57.15,
                                "name": "random_04",
                                "status": "RE",
                            },
                            {
                                "elapsed": 2.76,
                                "environment": "awk",
                                "memory": 6.76,
                                "name": "random_05",
                                "status": "RE",
                            },
                            {
                                "elapsed": 8.25,
                                "environment": "awk",
                                "memory": 23.66,
                                "name": "random_06",
                                "status": "RE",
                            },
                            {
                                "elapsed": 6.99,
                                "environment": "awk",
                                "memory": 66.49,
                                "name": "random_07",
                                "status": "RE",
                            },
                            {
                                "elapsed": 8.71,
                                "environment": "awk",
                                "memory": 16.96,
                                "name": "random_08",
                                "status": "RE",
                            },
                            {
                                "elapsed": 5.17,
                                "environment": "awk",
                                "memory": 36.25,
                                "name": "random_09",
                                "status": "RE",
                            },
                        ],
                        "timestamp": "1989-12-14 00:37:12.870000+00:00",
                        "verificationStatus": "TEST_WRONG_ANSWER",
                        "verifiedWith": [],
                    },
                    "documentation_of": "awk/aplusb.test.awk",
                    "layout": "document",
                },
            ),
            MarkdownData(
                path="awk/aplusb_direct.awk.md",
                front_matter={
                    "data": {
                        "attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb"
                        },
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [],
                        "embedded": [
                            {
                                "code": pathlib.Path("awk/aplusb_direct.awk").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": True,
                        "path": "awk/aplusb_direct.awk",
                        "pathExtension": "awk",
                        "requiredBy": [],
                        "testcases": [
                            {
                                "elapsed": 3.82,
                                "environment": "awk",
                                "memory": 48.12,
                                "name": "example_00",
                                "status": "AC",
                            },
                            {
                                "elapsed": 6.27,
                                "environment": "awk",
                                "memory": 69.16,
                                "name": "example_01",
                                "status": "AC",
                            },
                            {
                                "elapsed": 8.99,
                                "environment": "awk",
                                "memory": 97.03,
                                "name": "random_00",
                                "status": "AC",
                            },
                            {
                                "elapsed": 7.9,
                                "environment": "awk",
                                "memory": 25.28,
                                "name": "random_01",
                                "status": "AC",
                            },
                            {
                                "elapsed": 5.37,
                                "environment": "awk",
                                "memory": 77.2,
                                "name": "random_02",
                                "status": "AC",
                            },
                            {
                                "elapsed": 4.71,
                                "environment": "awk",
                                "memory": 95.31,
                                "name": "random_03",
                                "status": "AC",
                            },
                            {
                                "elapsed": 0.02,
                                "environment": "awk",
                                "memory": 3.83,
                                "name": "random_04",
                                "status": "AC",
                            },
                            {
                                "elapsed": 8.69,
                                "environment": "awk",
                                "memory": 55.61,
                                "name": "random_05",
                                "status": "AC",
                            },
                            {
                                "elapsed": 4.75,
                                "environment": "awk",
                                "memory": 46.8,
                                "name": "random_06",
                                "status": "AC",
                            },
                            {
                                "elapsed": 8.08,
                                "environment": "awk",
                                "memory": 75.38,
                                "name": "random_07",
                                "status": "AC",
                            },
                            {
                                "elapsed": 9.96,
                                "environment": "awk",
                                "memory": 58.54,
                                "name": "random_08",
                                "status": "AC",
                            },
                            {
                                "elapsed": 7.36,
                                "environment": "awk",
                                "memory": 98.49,
                                "name": "random_09",
                                "status": "AC",
                            },
                        ],
                        "timestamp": "2058-12-19 05:34:30.160000+04:00",
                        "verificationStatus": "TEST_ACCEPTED",
                        "verifiedWith": [],
                    },
                    "documentation_of": "awk/aplusb_direct.awk",
                    "layout": "document",
                },
            ),
            MarkdownData(
                path="encoding/EUC-KR.txt.md",
                front_matter={
                    "data": {
                        "attributes": {},
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "cp932.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "encoding/cp932.txt",
                                    }
                                ],
                                "type": "Depends on",
                            },
                            {
                                "files": [
                                    {
                                        "filename": "cp932.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "encoding/cp932.txt",
                                    }
                                ],
                                "type": "Required by",
                            },
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": ["encoding/cp932.txt"],
                        "embedded": [
                            {
                                "code": "컴퓨터 프로그램(영국 영어: computer programme, 미국 영어: computer program)은 컴퓨터에서 실행될 때 특정 작업(specific task)을 수행하는 일련의 명령어들의 모음(집합)이다.[1][2] 특정 문제를 해결하기 위해 처리 방법과 순서를 기술하여 컴퓨터에 입력되는 일련의 명령문 집합체이며 대부분의 프로그램은 실행 중(즉, 명령어를 '불러들일' 때)에 사용자의 입력에 반응하도록 구현된 일련의 명령어들로 구성되어 있다. 대부분의 프로그램들은 하드디스크 등의 매체에 바이너리 형식의 파일로 저장되어 있다가 사용자가 실행시키면 메모리로 적재되어 실행된다. 컴퓨터 소프트웨어와 비슷한 뜻을 가지고 있다. \"컴퓨터프로그램저작물\"은 저작권법상 저작물로서 보호된다. 동법에서 컴퓨터프로그램저작물이라 함은 특정한 결과를 얻기 위하여 컴퓨터 등 정보처리능력을 가진 장치 내에서 직접 또는 간접으로 사용되는 일련의 지시?· 명령으로 표현된 창작물을 말한다. (대한민국 저작권법 제2조 16호 및 제4조 제1항 9호)",
                                "name": "default",
                            },
                            {"code": f"cp949{os.linesep}", "name": "bundled"},
                        ],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "encoding/EUC-KR.txt",
                        "pathExtension": "txt",
                        "requiredBy": ["encoding/cp932.txt"],
                        "timestamp": "2049-12-17 18:54:29.650000+03:00",
                        "verificationStatus": "LIBRARY_NO_TESTS",
                        "verifiedWith": [],
                    },
                    "documentation_of": "encoding/EUC-KR.txt",
                    "layout": "document",
                },
            ),
            MarkdownData(
                path="encoding/cp932.txt.md",
                front_matter={
                    "data": {
                        "attributes": {},
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "EUC-KR.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "encoding/EUC-KR.txt",
                                    }
                                ],
                                "type": "Depends on",
                            },
                            {
                                "files": [
                                    {
                                        "filename": "EUC-KR.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "encoding/EUC-KR.txt",
                                    }
                                ],
                                "type": "Required by",
                            },
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": ["encoding/EUC-KR.txt"],
                        "embedded": [
                            {
                                "code": "雨ニモマケズ\n風ニモマケズ\n雪ニモ夏ノ暑サニモマケヌ\n丈夫ナカラダヲモチ\n慾ハナク\n決シテ瞋ラズ\nイツモシヅカニワラッテヰル\n一日ニ玄米四合ト\n味噌ト少シノ野菜ヲタベ\nアラユルコトヲ\nジブンヲカンジョウニ入レズニ\nヨクミキキシワカリ\nソシテワスレズ\n野原ﾉ松ﾉ林ﾉ陰ﾉ\n小ｻﾅ萱ﾌﾞｷﾉ小屋ﾆヰﾃ\n東ﾆ病気ﾉｺﾄﾞﾓｱﾚﾊﾞ\n行ｯﾃ看病ｼﾃﾔﾘ\n西ﾆﾂｶﾚﾀ母ｱﾚﾊﾞ\n行ｯﾃｿﾉ稲ﾉ束ｦ負ﾋ\n南ﾆ死ﾆｻｳﾅ人ｱﾚﾊﾞ\n行ｯﾃｺﾊｶﾞﾗﾅｸﾃﾓｲヽﾄｲﾋ\n北ﾆｹﾝｸヮﾔｿｼｮｳｶﾞｱﾚﾊﾞ\nﾂﾏﾗﾅｲｶﾗﾔﾒﾛﾄｲﾋ\nﾋﾃﾞﾘﾉﾄｷﾊﾅﾐﾀﾞｦﾅｶﾞｼ\nｻﾑｻﾉﾅﾂﾊｵﾛｵﾛｱﾙｷ\nﾐﾝﾅﾆﾃﾞｸﾉﾎﾞーﾄﾖﾊﾞﾚ\nﾎﾒﾗﾚﾓｾｽﾞ\nｸﾆﾓｻﾚｽﾞ\nｻｳｲﾌﾓﾉﾆ\nﾜﾀｼﾊﾅﾘﾀｲ",
                                "name": "default",
                            },
                            {"code": f"cp932{os.linesep}", "name": "bundled"},
                        ],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "encoding/cp932.txt",
                        "pathExtension": "txt",
                        "requiredBy": ["encoding/EUC-KR.txt"],
                        "timestamp": "2049-12-17 18:54:29.650000+03:00",
                        "verificationStatus": "LIBRARY_NO_TESTS",
                        "verifiedWith": [],
                    },
                    "documentation_of": "encoding/cp932.txt",
                    "layout": "document",
                },
            ),
            MarkdownData(
                path="python/failure.mle.py.md",
                front_matter={
                    "data": {
                        "attributes": {
                            "MLE": "100",
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "lib_all_failure.py",
                                        "icon": "LIBRARY_ALL_WA",
                                        "path": "python/lib_all_failure.py",
                                        "title": "Lib All failure",
                                    },
                                    {
                                        "filename": "lib_some_failure.py",
                                        "title": "Units📏",
                                        "icon": "LIBRARY_SOME_WA",
                                        "path": "python/lib_some_failure.py",
                                    },
                                ],
                                "type": "Depends on",
                            },
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [
                            "python/lib_all_failure.py",
                            "python/lib_some_failure.py",
                        ],
                        "documentPath": "python/sub/failure.mle.md",
                        "embedded": [
                            {
                                "code": pathlib.Path("python/failure.mle.py").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": True,
                        "isVerificationFile": True,
                        "path": "python/failure.mle.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "testcases": [
                            {
                                "name": "example_00",
                                "environment": "Python",
                                "status": "MLE",
                                "elapsed": 5.34,
                                "memory": 89.12,
                            },
                            {
                                "name": "example_01",
                                "environment": "Python",
                                "status": "MLE",
                                "elapsed": 9.79,
                                "memory": 78.31,
                            },
                            {
                                "name": "random_00",
                                "environment": "Python",
                                "status": "MLE",
                                "elapsed": 4.8,
                                "memory": 6.08,
                            },
                            {
                                "name": "random_01",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 4.09,
                                "memory": 15.08,
                            },
                            {
                                "name": "random_02",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 3.33,
                                "memory": 6.99,
                            },
                            {
                                "name": "random_03",
                                "environment": "Python",
                                "status": "MLE",
                                "elapsed": 4.04,
                                "memory": 9.04,
                            },
                            {
                                "name": "random_04",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 8.19,
                                "memory": 81.73,
                            },
                            {
                                "name": "random_05",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 0.18,
                                "memory": 47.13,
                            },
                            {
                                "name": "random_06",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 1.01,
                                "memory": 68.03,
                            },
                            {
                                "name": "random_07",
                                "environment": "Python",
                                "status": "MLE",
                                "elapsed": 3.76,
                                "memory": 40.93,
                            },
                            {
                                "name": "random_08",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 1.32,
                                "memory": 69.99,
                            },
                            {
                                "name": "random_09",
                                "environment": "Python",
                                "status": "MLE",
                                "elapsed": 0.12,
                                "memory": 19.27,
                            },
                        ]
                        if check_gnu_time()
                        else [],
                        "timestamp": "2063-11-24 03:09:17.740000+12:00",
                        "verificationStatus": "TEST_WRONG_ANSWER",
                        "verifiedWith": [],
                        "title": "Failure-MLE",
                    },
                    "documentation_of": "python/failure.mle.py",
                    "layout": "document",
                    "title": "Failure-MLE",
                },
            ),
            MarkdownData(
                path="python/failure.re.py.md",
                front_matter={
                    "data": {
                        "attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "lib_all_failure.py",
                                        "icon": "LIBRARY_ALL_WA",
                                        "path": "python/lib_all_failure.py",
                                        "title": "Lib All failure",
                                    }
                                ],
                                "type": "Depends on",
                            },
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": ["python/lib_all_failure.py"],
                        "documentPath": "failure.re.md",
                        "embedded": [
                            {
                                "code": pathlib.Path("python/failure.re.py").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": True,
                        "isVerificationFile": True,
                        "path": "python/failure.re.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "testcases": [
                            {
                                "name": "example_00",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 3.47,
                                "memory": 55.14,
                            },
                            {
                                "name": "example_01",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 4.8,
                                "memory": 82.74,
                            },
                            {
                                "name": "random_00",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 4.62,
                                "memory": 53.04,
                            },
                            {
                                "name": "random_01",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 5.6,
                                "memory": 84.32,
                            },
                            {
                                "name": "random_02",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 9.28,
                                "memory": 18.09,
                            },
                            {
                                "name": "random_03",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 6.41,
                                "memory": 13.18,
                            },
                            {
                                "name": "random_04",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 7.17,
                                "memory": 63.97,
                            },
                            {
                                "name": "random_05",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 7.59,
                                "memory": 75.35,
                            },
                            {
                                "name": "random_06",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 4.79,
                                "memory": 50.9,
                            },
                            {
                                "name": "random_07",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 2.83,
                                "memory": 25.32,
                            },
                            {
                                "name": "random_08",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 1.71,
                                "memory": 53.67,
                            },
                            {
                                "name": "random_09",
                                "environment": "Python",
                                "status": "RE",
                                "elapsed": 4.52,
                                "memory": 85.17,
                            },
                        ],
                        "timestamp": "2025-09-24 11:55:03.150000+03:00",
                        "verificationStatus": "TEST_WRONG_ANSWER",
                        "verifiedWith": [],
                        "title": "Failure-RE",
                    },
                    "documentation_of": "python/failure.re.py",
                    "layout": "document",
                    "title": "Failure-RE",
                },
            ),
            MarkdownData(
                path="python/failure.tle.py.md",
                front_matter={
                    "data": {
                        "attributes": {
                            "PROBLEM": "https://onlinejudge.u-aizu.ac.jp/courses/lesson/2/ITP1/1/ITP1_1_A",
                            "TLE": "0.1",
                            "links": [
                                "https://onlinejudge.u-aizu.ac.jp/courses/lesson/2/ITP1/1/ITP1_1_A"
                            ],
                        },
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "lib_all_failure.py",
                                        "icon": "LIBRARY_ALL_WA",
                                        "path": "python/lib_all_failure.py",
                                        "title": "Lib All failure",
                                    }
                                ],
                                "type": "Depends on",
                            },
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": ["python/lib_all_failure.py"],
                        "documentPath": "python/sub/failure.tle.md",
                        "embedded": [
                            {
                                "code": pathlib.Path("python/failure.tle.py").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": True,
                        "isVerificationFile": True,
                        "path": "python/failure.tle.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "testcases": [
                            {
                                "environment": "Python",
                                "name": "judge_data",
                                "status": "TLE",
                                "elapsed": 6.75,
                                "memory": 8.22,
                            },
                        ],
                        "timestamp": "2006-12-18 06:30:34.720000+10:00",
                        "verificationStatus": "TEST_WRONG_ANSWER",
                        "verifiedWith": [],
                        "title": "Failure-TLE",
                    },
                    "documentation_of": "python/failure.tle.py",
                    "layout": "document",
                    "title": "Failure-TLE",
                },
            ),
            MarkdownData(
                path="python/failure.wa.py.md",
                front_matter={
                    "data": {
                        "attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "lib_all_failure.py",
                                        "icon": "LIBRARY_ALL_WA",
                                        "path": "python/lib_all_failure.py",
                                        "title": "Lib All failure",
                                    },
                                    {
                                        "filename": "lib_some_skip_some_wa.py",
                                        "icon": "LIBRARY_SOME_WA",
                                        "path": "python/lib_some_skip_some_wa.py",
                                    },
                                ],
                                "type": "Depends on",
                            },
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [
                            "python/lib_all_failure.py",
                            "python/lib_some_skip_some_wa.py",
                        ],
                        "documentPath": "failure.wa.md",
                        "embedded": [
                            {
                                "code": pathlib.Path("python/failure.wa.py").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": True,
                        "isVerificationFile": True,
                        "path": "python/failure.wa.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "testcases": [
                            {
                                "name": "example_00",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 1.89,
                                "memory": 34.41,
                            },
                            {
                                "name": "example_01",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 2.17,
                                "memory": 10.24,
                            },
                            {
                                "name": "random_00",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 3.25,
                                "memory": 7.82,
                            },
                            {
                                "name": "random_01",
                                "environment": "Python",
                                "status": "WA",
                                "elapsed": 7.41,
                                "memory": 81.13,
                            },
                            {
                                "name": "random_02",
                                "environment": "Python",
                                "status": "WA",
                                "elapsed": 1.53,
                                "memory": 91.42,
                            },
                            {
                                "name": "random_03",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 4.6,
                                "memory": 13.13,
                            },
                            {
                                "name": "random_04",
                                "environment": "Python",
                                "status": "WA",
                                "elapsed": 5.09,
                                "memory": 38.65,
                            },
                            {
                                "name": "random_05",
                                "environment": "Python",
                                "status": "WA",
                                "elapsed": 6.34,
                                "memory": 73.13,
                            },
                            {
                                "name": "random_06",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 3.13,
                                "memory": 32.18,
                            },
                            {
                                "name": "random_07",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 8.88,
                                "memory": 10.07,
                            },
                            {
                                "name": "random_08",
                                "environment": "Python",
                                "status": "WA",
                                "elapsed": 1.66,
                                "memory": 88.91,
                            },
                            {
                                "name": "random_09",
                                "environment": "Python",
                                "status": "WA",
                                "elapsed": 9.41,
                                "memory": 34.78,
                            },
                        ],
                        "timestamp": "1987-12-04 11:42:47.800000-07:00",
                        "verificationStatus": "TEST_WRONG_ANSWER",
                        "verifiedWith": [],
                        "title": "Failure-WA",
                    },
                    "documentation_of": "python/failure.wa.py",
                    "layout": "document",
                    "title": "Failure-WA",
                },
            ),
            MarkdownData(
                path="python/lib_all_failure.py.md",
                content=b"\n# Lib All failure",
                front_matter={
                    "data": {
                        "attributes": {"links": []},
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {
                                "files": [
                                    {
                                        "filename": "failure.mle.py",
                                        "icon": "TEST_WRONG_ANSWER",
                                        "path": "python/failure.mle.py",
                                        "title": "Failure-MLE",
                                    },
                                    {
                                        "filename": "failure.re.py",
                                        "icon": "TEST_WRONG_ANSWER",
                                        "path": "python/failure.re.py",
                                        "title": "Failure-RE",
                                    },
                                    {
                                        "filename": "failure.tle.py",
                                        "icon": "TEST_WRONG_ANSWER",
                                        "path": "python/failure.tle.py",
                                        "title": "Failure-TLE",
                                    },
                                    {
                                        "filename": "failure.wa.py",
                                        "icon": "TEST_WRONG_ANSWER",
                                        "path": "python/failure.wa.py",
                                        "title": "Failure-WA",
                                    },
                                ],
                                "type": "Verified with",
                            },
                        ],
                        "dependsOn": [],
                        "documentPath": "python/docs_lib_all_failure.md",
                        "embedded": [
                            {
                                "code": pathlib.Path(
                                    "python/lib_all_failure.py"
                                ).read_text(encoding="utf-8"),
                                "name": "default",
                            }
                        ],
                        "isFailed": True,
                        "isVerificationFile": False,
                        "path": "python/lib_all_failure.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "timestamp": "1974-05-03 17:27:38.760000-11:00",
                        "verificationStatus": "LIBRARY_ALL_WA",
                        "verifiedWith": [
                            "python/failure.mle.py",
                            "python/failure.re.py",
                            "python/failure.tle.py",
                            "python/failure.wa.py",
                        ],
                        "title": "Lib All failure",
                    },
                    "documentation_of": "python/lib_all_failure.py",
                    "layout": "document",
                    "title": "Lib All failure",
                },
            ),
            MarkdownData(
                path="python/lib_all_success.py.md",
                content=b"\n# Lib All Success",
                front_matter={
                    "data": {
                        "attributes": {"links": []},
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {
                                "files": [
                                    {
                                        "filename": "success2.py",
                                        "icon": "TEST_ACCEPTED",
                                        "path": "python/success2.py",
                                        "title": "Success2",
                                    }
                                ],
                                "type": "Verified with",
                            },
                        ],
                        "dependsOn": [],
                        "documentPath": "python/docs_lib_all_success.md",
                        "embedded": [
                            {
                                "code": pathlib.Path(
                                    "python/lib_all_success.py"
                                ).read_text(encoding="utf-8"),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "python/lib_all_success.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "timestamp": "2059-10-27 14:16:37.850000-02:00",
                        "verificationStatus": "LIBRARY_ALL_AC",
                        "verifiedWith": ["python/success2.py"],
                        "title": "Lib All Success",
                    },
                    "documentation_of": "python/lib_all_success.py",
                    "layout": "document",
                    "title": "Lib All Success",
                },
            ),
            MarkdownData(
                path="python/lib_hidden.py.md",
                front_matter={
                    "data": {
                        "attributes": {"DISPLAY": "hidden", "links": []},
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [],
                        "embedded": [
                            {
                                "code": pathlib.Path("python/lib_hidden.py").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "python/lib_hidden.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "timestamp": "2050-01-17 11:34:55.500000-12:00",
                        "verificationStatus": "LIBRARY_NO_TESTS",
                        "verifiedWith": [],
                    },
                    "display": "hidden",
                    "documentation_of": "python/lib_hidden.py",
                    "layout": "document",
                },
            ),
            MarkdownData(
                path="python/lib_skip.py.md",
                content=b"\n# Skip",
                front_matter={
                    "data": {
                        "attributes": {"links": []},
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {
                                "files": [
                                    {
                                        "filename": "skip.py",
                                        "icon": "TEST_WAITING_JUDGE",
                                        "path": "python/skip.py",
                                    }
                                ],
                                "type": "Verified with",
                            },
                        ],
                        "dependsOn": [],
                        "documentPath": "python/docs_lib_skip.md",
                        "embedded": [
                            {
                                "code": pathlib.Path("python/lib_skip.py").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "python/lib_skip.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "timestamp": "1992-02-27 21:48:42.830000-04:00",
                        "verificationStatus": "LIBRARY_NO_TESTS",
                        "verifiedWith": ["python/skip.py"],
                        "title": "Skip Library",
                    },
                    "documentation_of": "python/lib_skip.py",
                    "layout": "document",
                    "title": "Skip Library",
                },
            ),
            MarkdownData(
                path="python/lib_some_failure.py.md",
                front_matter={
                    "data": {
                        "attributes": {"TITLE": "Units📏", "links": []},
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {
                                "files": [
                                    {
                                        "filename": "failure.mle.py",
                                        "icon": "TEST_WRONG_ANSWER",
                                        "path": "python/failure.mle.py",
                                        "title": "Failure-MLE",
                                    },
                                    {
                                        "filename": "success1.py",
                                        "icon": "TEST_ACCEPTED",
                                        "path": "python/success1.py",
                                        "title": "Success1",
                                    },
                                ],
                                "type": "Verified with",
                            },
                        ],
                        "dependsOn": [],
                        "embedded": [
                            {
                                "code": pathlib.Path(
                                    "python/lib_some_failure.py"
                                ).read_text(encoding="utf-8"),
                                "name": "default",
                            }
                        ],
                        "isFailed": True,
                        "isVerificationFile": False,
                        "path": "python/lib_some_failure.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "timestamp": "1979-10-23 04:53:31.440000+07:00",
                        "verificationStatus": "LIBRARY_SOME_WA",
                        "verifiedWith": [
                            "python/failure.mle.py",
                            "python/success1.py",
                        ],
                        "title": "Units📏",
                    },
                    "documentation_of": "python/lib_some_failure.py",
                    "layout": "document",
                    "title": "Units📏",
                },
            ),
            MarkdownData(
                path="python/lib_some_skip.py.md",
                front_matter={
                    "data": {
                        "attributes": {"links": []},
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {
                                "files": [
                                    {
                                        "filename": "success1.py",
                                        "icon": "TEST_ACCEPTED",
                                        "path": "python/success1.py",
                                        "title": "Success1",
                                    }
                                ],
                                "type": "Verified with",
                            },
                        ],
                        "dependsOn": [],
                        "embedded": [
                            {
                                "code": pathlib.Path(
                                    "python/lib_some_skip.py"
                                ).read_text(encoding="utf-8"),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "python/lib_some_skip.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "timestamp": "2014-03-20 04:17:19.730000+11:00",
                        "verificationStatus": "LIBRARY_ALL_AC",
                        "verifiedWith": ["python/success1.py"],
                    },
                    "documentation_of": "python/lib_some_skip.py",
                    "layout": "document",
                },
            ),
            MarkdownData(
                path="python/lib_some_skip_some_wa.py.md",
                front_matter={
                    "data": {
                        "attributes": {"links": []},
                        "dependencies": [
                            {"files": [], "type": "Depends on"},
                            {"files": [], "type": "Required by"},
                            {
                                "files": [
                                    {
                                        "filename": "failure.wa.py",
                                        "icon": "TEST_WRONG_ANSWER",
                                        "path": "python/failure.wa.py",
                                        "title": "Failure-WA",
                                    },
                                    {
                                        "filename": "skip.py",
                                        "icon": "TEST_WAITING_JUDGE",
                                        "path": "python/skip.py",
                                    },
                                    {
                                        "filename": "success2.py",
                                        "icon": "TEST_ACCEPTED",
                                        "path": "python/success2.py",
                                        "title": "Success2",
                                    },
                                ],
                                "type": "Verified with",
                            },
                        ],
                        "dependsOn": [],
                        "embedded": [
                            {
                                "code": pathlib.Path(
                                    "python/lib_some_skip_some_wa.py"
                                ).read_text(encoding="utf-8"),
                                "name": "default",
                            }
                        ],
                        "isFailed": True,
                        "isVerificationFile": False,
                        "path": "python/lib_some_skip_some_wa.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "timestamp": "2040-07-11 20:28:24.310000-06:00",
                        "verificationStatus": "LIBRARY_SOME_WA",
                        "verifiedWith": [
                            "python/failure.wa.py",
                            "python/skip.py",
                            "python/success2.py",
                        ],
                    },
                    "documentation_of": "python/lib_some_skip_some_wa.py",
                    "layout": "document",
                },
            ),
            MarkdownData(
                path="python/skip.py.md",
                front_matter={
                    "data": {
                        "attributes": {
                            "IGNORE": "",
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "lib_skip.py",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "python/lib_skip.py",
                                        "title": "Skip Library",
                                    },
                                    {
                                        "filename": "lib_some_skip_some_wa.py",
                                        "icon": "LIBRARY_SOME_WA",
                                        "path": "python/lib_some_skip_some_wa.py",
                                    },
                                ],
                                "type": "Depends on",
                            },
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [
                            "python/lib_skip.py",
                            "python/lib_some_skip_some_wa.py",
                        ],
                        "embedded": [
                            {
                                "code": pathlib.Path("python/skip.py").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": True,
                        "path": "python/skip.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "testcases": [],
                        "timestamp": "2027-02-03 19:13:12.050000-07:00",
                        "verificationStatus": "TEST_WAITING_JUDGE",
                        "verifiedWith": [],
                    },
                    "documentation_of": "python/skip.py",
                    "layout": "document",
                },
            ),
            MarkdownData(
                path="python/success1.py.md",
                front_matter={
                    "data": {
                        "attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "lib_some_failure.py",
                                        "title": "Units📏",
                                        "icon": "LIBRARY_SOME_WA",
                                        "path": "python/lib_some_failure.py",
                                    },
                                    {
                                        "filename": "lib_some_skip.py",
                                        "icon": "LIBRARY_ALL_AC",
                                        "path": "python/lib_some_skip.py",
                                    },
                                ],
                                "type": "Depends on",
                            },
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [
                            "python/lib_some_failure.py",
                            "python/lib_some_skip.py",
                        ],
                        "documentPath": "python/docs_success1.md",
                        "embedded": [
                            {
                                "code": pathlib.Path("python/success1.py").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": True,
                        "path": "python/success1.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "testcases": [
                            {
                                "name": "example_00",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 8.66,
                                "memory": 14.97,
                            },
                            {
                                "name": "example_01",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 3.4,
                                "memory": 82.48,
                            },
                            {
                                "name": "random_00",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 6.01,
                                "memory": 69.23,
                            },
                            {
                                "name": "random_01",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 8.47,
                                "memory": 16.15,
                            },
                            {
                                "name": "random_02",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 1.28,
                                "memory": 88.6,
                            },
                            {
                                "name": "random_03",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 5.69,
                                "memory": 85.15,
                            },
                            {
                                "name": "random_04",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 2.52,
                                "memory": 74.99,
                            },
                            {
                                "name": "random_05",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 2.0,
                                "memory": 4.66,
                            },
                            {
                                "name": "random_06",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 4.46,
                                "memory": 13.48,
                            },
                            {
                                "name": "random_07",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 7.96,
                                "memory": 6.91,
                            },
                            {
                                "name": "random_08",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 7.67,
                                "memory": 39.47,
                            },
                            {
                                "name": "random_09",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 0.16,
                                "memory": 36.45,
                            },
                        ],
                        "timestamp": "1972-12-09 20:42:07.860000-01:00",
                        "verificationStatus": "TEST_ACCEPTED",
                        "verifiedWith": [],
                        "title": "Success1",
                    },
                    "documentation_of": "python/success1.py",
                    "layout": "document",
                    "title": "Success1",
                },
            ),
            MarkdownData(
                path="python/success2.py.md",
                front_matter={
                    "data": {
                        "attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "lib_all_success.py",
                                        "icon": "LIBRARY_ALL_AC",
                                        "path": "python/lib_all_success.py",
                                        "title": "Lib All Success",
                                    },
                                    {
                                        "filename": "lib_some_skip_some_wa.py",
                                        "icon": "LIBRARY_SOME_WA",
                                        "path": "python/lib_some_skip_some_wa.py",
                                    },
                                ],
                                "type": "Depends on",
                            },
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [
                            "python/lib_all_success.py",
                            "python/lib_some_skip_some_wa.py",
                        ],
                        "documentPath": "python/docs_success2.md",
                        "embedded": [
                            {
                                "code": pathlib.Path("python/success2.py").read_text(
                                    encoding="utf-8"
                                ),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": True,
                        "path": "python/success2.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "testcases": [
                            {
                                "name": "example_00",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 1.1,
                                "memory": 53.93,
                            },
                            {
                                "name": "example_01",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 5.48,
                                "memory": 50.39,
                            },
                            {
                                "name": "random_00",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 9.31,
                                "memory": 58.92,
                            },
                            {
                                "name": "random_01",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 4.3,
                                "memory": 35.83,
                            },
                            {
                                "name": "random_02",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 2.64,
                                "memory": 87.34,
                            },
                            {
                                "name": "random_03",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 2.73,
                                "memory": 54.56,
                            },
                            {
                                "name": "random_04",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 5.98,
                                "memory": 34.29,
                            },
                            {
                                "name": "random_05",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 3.52,
                                "memory": 91.97,
                            },
                            {
                                "name": "random_06",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 7.15,
                                "memory": 79.87,
                            },
                            {
                                "name": "random_07",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 6.63,
                                "memory": 11.11,
                            },
                            {
                                "name": "random_08",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 6.53,
                                "memory": 74.19,
                            },
                            {
                                "name": "random_09",
                                "environment": "Python",
                                "status": "AC",
                                "elapsed": 7.4,
                                "memory": 27.16,
                            },
                        ],
                        "timestamp": "1977-03-05 16:44:55.840000-03:00",
                        "verificationStatus": "TEST_ACCEPTED",
                        "verifiedWith": [],
                        "title": "Success2",
                    },
                    "documentation_of": "python/success2.py",
                    "layout": "document",
                    "title": "Success2",
                },
            ),
            MarkdownData(
                path="python/sub/no_dependants.py.md",
                front_matter={
                    "data": {
                        "attributes": {"links": []},
                        "dependencies": [
                            {
                                "files": [],
                                "type": "Depends on",
                            },
                            {"files": [], "type": "Required by"},
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [],
                        "embedded": [
                            {
                                "code": pathlib.Path(
                                    "python/sub/no_dependants.py"
                                ).read_text(encoding="utf-8"),
                                "name": "default",
                            }
                        ],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "python/sub/no_dependants.py",
                        "pathExtension": "py",
                        "requiredBy": [],
                        "timestamp": "2050-11-28 04:31:04.040000-08:00",
                        "verificationStatus": "LIBRARY_NO_TESTS",
                        "verifiedWith": [],
                    },
                    "documentation_of": "python/sub/no_dependants.py",
                    "layout": "document",
                },
            ),
        ],
    )


@pytest.fixture
def package_dst(file_paths: FilePaths):
    return file_paths.dest_root / "documents"


@pytest.fixture
def setup_docs(mocker: MockerFixture):
    mocker.patch.dict(
        os.environ,
        {
            "GITHUB_REF_NAME": "TESTING_GIT_REF",
            "GITHUB_WORKFLOW": "TESTING_WORKFLOW",
        },
    )


def check_common(
    destination: pathlib.Path,
    *,
    data: DocsData,
    subtests: SubTests,
):
    assert destination.is_dir()

    targets = {t.path: t for t in data.targets_data}

    assert not (destination / "dummy").exists()

    for target_file in filter(
        lambda p: p.is_file(),
        chain.from_iterable(
            (
                (destination / "awk").glob("**/*"),
                (destination / "encoding").glob("**/*"),
                (destination / "python").glob("**/*"),
            )
        ),
    ):
        path_str = target_file.relative_to(destination).as_posix()
        with subtests.test(msg=path_str):  # pyright: ignore[reportUnknownMemberType]
            front_matter, content = split_front_matter_raw(target_file.read_bytes())
            assert content == targets[path_str].content
            assert front_matter
            assert yaml.safe_load(front_matter) == targets[path_str].front_matter
        del targets[path_str]

    assert not list(targets.keys())


@pytest.mark.integration
@pytest.mark.usefixtures("additional_path")
@pytest.mark.order(-100)
class TestCommandDocuments:
    @pytest.mark.usefixtures("setup_docs")
    def test_with_config(
        self,
        user_defined_and_python_data: UserDefinedAndPythonData,
        package_dst: pathlib.Path,
        data: DocsData,
        subtests: SubTests,
    ):
        destination = package_dst / inspect.stack()[0].function
        docs_settings_dir = user_defined_and_python_data.targets_path / "docs_settings"

        main(
            [
                "--docs",
                docs_settings_dir.as_posix(),
                "--destination",
                destination.as_posix(),
            ]
            + data.default_args
        )

        check_common(destination, data=data, subtests=subtests)

        config_yml = yaml.safe_load((destination / "_config.yml").read_bytes())
        assert config_yml == {
            "action_name": "TESTING_WORKFLOW",
            "basedir": "integration_test_data/UserDefinedAndPythonData/",
            "branch_name": "TESTING_GIT_REF",
            "description": "My description",
            "filename-index": True,
            "highlightjs-style": "vs2015",
            "plugins": [
                "jemoji",
                "jekyll-redirect-from",
                "jekyll-remote-theme",
            ],
            "mathjax": 2,
            "sass": {"style": "compressed"},
            "theme": "jekyll-theme-modernist",
            "icons": {
                "LIBRARY_ALL_AC": ":heavy_check_mark:",
                "LIBRARY_ALL_WA": ":x:",
                "LIBRARY_NO_TESTS": ":warning:",
                "LIBRARY_PARTIAL_AC": ":heavy_check_mark:",
                "LIBRARY_SOME_WA": ":question:",
                "TEST_ACCEPTED": ":100:",
                "TEST_WAITING_JUDGE": ":warning:",
                "TEST_WRONG_ANSWER": ":x:",
            },
        }

        assert (destination / "static.md").exists()
        assert (destination / "static.md").read_text(
            encoding="utf-8"
        ) == "# Static page\n\nI'm Static!"

        static_dir = docs_settings_dir / "static"
        for static_file in filter(lambda p: p.is_file(), static_dir.glob("**/*")):
            assert filecmp.cmp(
                static_file, destination / static_file.relative_to(static_dir)
            )

    @pytest.mark.usefixtures("setup_docs")
    def test_without_config(
        self,
        package_dst: pathlib.Path,
        data: DocsData,
        subtests: SubTests,
    ):
        destination = package_dst / inspect.stack()[0].function

        main(
            [
                "--destination",
                destination.as_posix(),
            ]
            + data.default_args
        )

        check_common(destination, data=data, subtests=subtests)

        config_yml = yaml.safe_load((destination / "_config.yml").read_bytes())
        assert config_yml == {
            "action_name": "TESTING_WORKFLOW",
            "basedir": "integration_test_data/UserDefinedAndPythonData/",
            "branch_name": "TESTING_GIT_REF",
            "description": '<small>This documentation is automatically generated by <a href="https://github.com/competitive-verifier/competitive-verifier">competitive-verifier/competitive-verifier</a></small>',
            "filename-index": False,
            "highlightjs-style": "default",
            "plugins": [
                "jemoji",
                "jekyll-redirect-from",
                "jekyll-remote-theme",
            ],
            "mathjax": 3,
            "sass": {"style": "compressed"},
            "theme": "jekyll-theme-minimal",
            "icons": {
                "LIBRARY_ALL_AC": ":heavy_check_mark:",
                "LIBRARY_ALL_WA": ":x:",
                "LIBRARY_NO_TESTS": ":warning:",
                "LIBRARY_PARTIAL_AC": ":heavy_check_mark:",
                "LIBRARY_SOME_WA": ":question:",
                "TEST_ACCEPTED": ":heavy_check_mark:",
                "TEST_WAITING_JUDGE": ":warning:",
                "TEST_WRONG_ANSWER": ":x:",
            },
        }

        assert not (destination / "static.md").exists()

        resource_dir = pathlib.Path("src/competitive_verifier_resources/jekyll")
        for resource_file in filter(lambda p: p.is_file(), resource_dir.glob("**/*")):
            assert filecmp.cmp(
                resource_file, destination / resource_file.relative_to(resource_dir)
            )

    @pytest.mark.usefixtures("setup_docs")
    def test_logging_default(
        self,
        package_dst: pathlib.Path,
        data: DocsData,
        subtests: SubTests,
        caplog: pytest.LogCaptureFixture,
    ):
        caplog.set_level(logging.WARNING)
        destination = package_dst / inspect.stack()[0].function

        main(
            [
                "--destination",
                destination.as_posix(),
            ]
            + data.default_args
        )

        check_common(destination, data=data, subtests=subtests)

        assert not caplog.record_tuples

    @pytest.mark.usefixtures("setup_docs")
    def test_logging_include(
        self,
        package_dst: pathlib.Path,
        data: DocsData,
        subtests: SubTests,
        caplog: pytest.LogCaptureFixture,
    ):
        caplog.set_level(logging.WARNING)
        destination = package_dst / inspect.stack()[0].function

        main(
            [
                "--include",
                "python/",
                "encoding",
                "a*",
                "failure.*.md",
                "--destination",
                destination.as_posix(),
            ]
            + data.default_args
        )

        check_common(destination, data=data, subtests=subtests)

        assert not caplog.record_tuples

    @pytest.mark.parametrize(
        "exclude",
        [
            ["dummy/"],
            ["dummy/dummy.md"],
        ],
    )
    @pytest.mark.usefixtures("setup_docs")
    def test_logging_exclude(
        self,
        exclude: list[str],
        package_dst: pathlib.Path,
        data: DocsData,
        subtests: SubTests,
        caplog: pytest.LogCaptureFixture,
    ):
        caplog.set_level(logging.WARNING)
        destination = package_dst / inspect.stack()[0].function

        main(
            [
                "--exclude",
                *exclude,
                "--destination",
                destination.as_posix(),
            ]
            + data.default_args
        )

        check_common(destination, data=data, subtests=subtests)

        assert not caplog.record_tuples

    @pytest.mark.usefixtures("setup_docs")
    def test_logging_exclude_code_but_include_docs(
        self,
        package_dst: pathlib.Path,
        data: DocsData,
        subtests: SubTests,
        caplog: pytest.LogCaptureFixture,
    ):
        caplog.set_level(logging.WARNING)
        destination = package_dst / inspect.stack()[0].function

        main(
            [
                "--exclude",
                "dummy/dummy.py",
                "--destination",
                destination.as_posix(),
            ]
            + data.default_args
        )

        check_common(destination, data=data, subtests=subtests)

        assert caplog.record_tuples == [
            (
                "competitive_verifier.documents.render",
                30,
                "Markdown(dummy/dummy.md) documentation_of: ./dummy.py is not found.",
            ),
        ]


@pytest.mark.integration
@pytest.mark.usefixtures("additional_path")
@pytest.mark.usefixtures("setup_docs")
@pytest.mark.order(-100)
def test_hand_docs(
    monkeypatch: pytest.MonkeyPatch,
    file_paths: FilePaths,
    package_dst: pathlib.Path,
    subtests: SubTests,
):
    targets = file_paths.root / "HandMadeDocs"
    monkeypatch.chdir(targets)

    destination = package_dst / "handmade" / inspect.stack()[0].function

    main(
        [
            "--exclude",
            "docs/",
            "--docs",
            "docs/",
            "--destination",
            destination.as_posix(),
            "--verify-json",
            str(targets / "verify.json"),
            str(targets / "result.json"),
        ]
    )

    assert (destination / "_config.yml").exists()
    with (destination / "_config.yml").open("rb") as fp:
        config_yml = yaml.safe_load(fp)
    assert config_yml == {
        "action_name": "TESTING_WORKFLOW",
        "basedir": "integration_test_data/HandMadeDocs/",
        "branch_name": "TESTING_GIT_REF",
        "description": '<small>This documentation is automatically generated by <a href="https://github.com/competitive-verifier/competitive-verifier">competitive-verifier/competitive-verifier</a></small>',
        "filename-index": True,
        "highlightjs-style": "default",
        "icons": {
            "LIBRARY_ALL_AC": ":100:",
            "LIBRARY_ALL_WA": ":x:",
            "LIBRARY_NO_TESTS": ":warning:",
            "LIBRARY_PARTIAL_AC": ":heavy_check_mark:",
            "LIBRARY_SOME_WA": ":question:",
            "TEST_ACCEPTED": ":heavy_check_mark:",
            "TEST_WAITING_JUDGE": ":warning:",
            "TEST_WRONG_ANSWER": ":x:",
        },
        "mathjax": 3,
        "plugins": ["jemoji", "jekyll-redirect-from", "jekyll-remote-theme"],
        "remote_theme": "vaibhavvikas/jekyll-theme-minimalistic",
        "sass": {"style": "compressed"},
        "theme": "jekyll-theme-minimal",
        "consolidate": ["consolidate", "consolidate/a"],
        "additional-data": "`code`",
    }

    assert config_yml == ConfigYaml(
        basedir=pathlib.Path("integration_test_data/HandMadeDocs"),
        action_name="TESTING_WORKFLOW",
        branch_name="TESTING_GIT_REF",
        icons=ConfigIcons(LIBRARY_ALL_AC=":100:"),
        filename_index=True,
        consolidate={
            pathlib.Path("consolidate/"),
            pathlib.Path("consolidate/a/"),
        },
    ).model_copy(
        update={
            "additional-data": "`code`",
            "remote_theme": "vaibhavvikas/jekyll-theme-minimalistic",
        }
    ).model_dump(
        mode="json",
        exclude_none=True,
        by_alias=True,
    )

    assert not pathlib.Path(destination / "display/never.txt.md").exists()
    assert pathlib.Path(destination / "foo/bar.js").exists()
    assert not pathlib.Path(destination / "docs").exists()

    class TextFileData(NamedTuple):
        path: str
        update_data: dict[str, Any] = {}
        update_root: dict[str, Any] = {}
        content: bytes = b""

    text_files: list[TextFileData] = [
        TextFileData("a/b/c.txt"),
        TextFileData("root.txt"),
        TextFileData("consolidate/a/b/1.txt"),
        TextFileData("consolidate/a/b/2.txt"),
        TextFileData("consolidate/a/1.txt"),
        TextFileData("consolidate/a/2.txt"),
        TextFileData("consolidate/z/b/1.txt"),
        TextFileData("consolidate/z/b/2.txt"),
        TextFileData("consolidate/z/1.txt"),
        TextFileData("consolidate/z/2.txt"),
        TextFileData("consolidate/1.txt"),
        TextFileData("consolidate/2.txt"),
        TextFileData(
            "display/multi-no-keep.txt",
            {
                "attributes": {"TITLE": "Multi no keep $1$"},
                "dependencies": [
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep2.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep2.txt",
                                "title": "Multi no keep $2$",
                            }
                        ],
                        "type": "Depends on",
                    },
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep2.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep2.txt",
                                "title": "Multi no keep $2$",
                            },
                            {
                                "filename": "no-index.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/no-index.txt",
                                "title": "display=no-index",
                            },
                            {
                                "filename": "visible.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/visible.txt",
                                "title": "display=visible",
                            },
                        ],
                        "type": "Required by",
                    },
                    {"files": [], "type": "Verified with"},
                ],
                "dependsOn": ["display/multi-no-keep2.txt"],
                "requiredBy": [
                    "display/multi-no-keep2.txt",
                    "display/no-index.txt",
                    "display/visible.txt",
                ],
                "timestamp": "2017-12-03 09:56:02.050000-07:00",
                "title": "Multi no keep $1$",
            },
            {
                "display": "no-index",
                "redirect_to": "/display/multi-no-keep",
                "title": "Multi no keep $1$",
            },
        ),
        TextFileData(
            "display/multi-no-keep2.txt",
            {
                "dependencies": [
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep.txt",
                                "title": "Multi no keep $1$",
                            }
                        ],
                        "type": "Depends on",
                    },
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep.txt",
                                "title": "Multi no keep $1$",
                            }
                        ],
                        "type": "Required by",
                    },
                    {"files": [], "type": "Verified with"},
                ],
                "documentPath": "display/multi-no-keep2.md",
                "dependsOn": ["display/multi-no-keep.txt"],
                "requiredBy": ["display/multi-no-keep.txt"],
                "timestamp": "2017-12-03 09:56:02.050000-07:00",
                "title": "Multi no keep $2$",
            },
            {
                "display": "no-index",
                "redirect_to": "/display/multi-no-keep",
                "title": "Multi no keep $2$",
            },
            b"\nKeep $2^x$",
        ),
        TextFileData(
            "display/hidden.txt",
            {
                "documentPath": "display/hidden.md",
                "timestamp": "2029-10-02 13:51:10.570000-05:00",
                "dependsOn": [
                    "display/multi-no-keep.txt",
                    "display/no-index.txt",
                    "display/visible.txt",
                ],
                "requiredBy": ["display/no-index.txt", "display/visible.txt"],
                "dependencies": [
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep.txt",
                                "title": "Multi no keep $1$",
                            },
                            {
                                "filename": "no-index.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/no-index.txt",
                                "title": "display=no-index",
                            },
                            {
                                "filename": "visible.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/visible.txt",
                                "title": "display=visible",
                            },
                        ],
                        "type": "Depends on",
                    },
                    {
                        "files": [
                            {
                                "filename": "no-index.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/no-index.txt",
                                "title": "display=no-index",
                            },
                            {
                                "filename": "visible.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/visible.txt",
                                "title": "display=visible",
                            },
                        ],
                        "type": "Required by",
                    },
                    {"files": [], "type": "Verified with"},
                ],
                "title": "display=hidden",
            },
            {
                "display": "hidden",
                "title": "display=hidden",
            },
        ),
        TextFileData(
            "display/no-index.txt",
            {
                "documentPath": "display/no-index.md",
                "timestamp": "2029-10-02 13:51:10.570000-05:00",
                "dependsOn": ["display/multi-no-keep.txt", "display/visible.txt"],
                "requiredBy": ["display/visible.txt"],
                "dependencies": [
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep.txt",
                                "title": "Multi no keep $1$",
                            },
                            {
                                "filename": "visible.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/visible.txt",
                                "title": "display=visible",
                            },
                        ],
                        "type": "Depends on",
                    },
                    {
                        "files": [
                            {
                                "filename": "visible.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/visible.txt",
                                "title": "display=visible",
                            },
                        ],
                        "type": "Required by",
                    },
                    {"files": [], "type": "Verified with"},
                ],
                "title": "display=no-index",
            },
            {
                "display": "no-index",
                "title": "display=no-index",
            },
        ),
        TextFileData(
            "display/visible.txt",
            {
                "timestamp": "2029-10-02 13:51:10.570000-05:00",
                "documentPath": "display/visible.md",
                "dependsOn": [
                    "display/multi-no-keep.txt",
                    "display/no-index.txt",
                ],
                "requiredBy": ["display/no-index.txt"],
                "dependencies": [
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep.txt",
                                "title": "Multi no keep $1$",
                            },
                            {
                                "filename": "no-index.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/no-index.txt",
                                "title": "display=no-index",
                            },
                        ],
                        "type": "Depends on",
                    },
                    {
                        "files": [
                            {
                                "filename": "no-index.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/no-index.txt",
                                "title": "display=no-index",
                            },
                        ],
                        "type": "Required by",
                    },
                    {"files": [], "type": "Verified with"},
                ],
                "title": "display=visible",
            },
            {
                "display": "visible",
                "title": "display=visible",
            },
            b"# Visible",
        ),
        TextFileData(
            "display/no-index2.txt",
            {"attributes": {"DISPLAY": "no-index"}},
            {"display": "no-index"},
        ),
        TextFileData(
            "display/no-index3.txt",
            {"attributes": {"DISPLAY": "no_index"}},
            {"display": "no-index"},
        ),
        TextFileData("txts/utf-8👍.txt"),
        TextFileData(
            "txts/utf-16BE.txt",
            {
                "attributes": {"document_title": "UTF-16BE"},
                "timestamp": "1974-07-18 21:35:09.220000+10:00",
                "embedded": [
                    {
                        "code": "🐴🐑🐵🐔🐶🐗",
                        "name": "default",
                    }
                ],
                "dependencies": [
                    {
                        "files": [
                            {
                                "filename": "utf-16LE.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "txts/utf-16LE.txt",
                                "title": "UTF-16LE",
                            }
                        ],
                        "type": "Depends on",
                    },
                    {
                        "files": [
                            {
                                "filename": "utf-16LE.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "txts/utf-16LE.txt",
                                "title": "UTF-16LE",
                            }
                        ],
                        "type": "Required by",
                    },
                    {"files": [], "type": "Verified with"},
                ],
                "dependsOn": ["txts/utf-16LE.txt"],
                "requiredBy": ["txts/utf-16LE.txt"],
                "title": "UTF-16BE",
            },
            {
                "title": "UTF-16BE",
            },
        ),
        TextFileData(
            "txts/utf-16LE.txt",
            {
                "attributes": {"TITLE": "UTF-16LE"},
                "timestamp": "1974-07-18 21:35:09.220000+10:00",
                "embedded": [
                    {
                        "code": "🐭🐮🐯🐰🐲🐍",
                        "name": "default",
                    }
                ],
                "dependencies": [
                    {
                        "files": [
                            {
                                "filename": "utf-16BE.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "txts/utf-16BE.txt",
                                "title": "UTF-16BE",
                            }
                        ],
                        "type": "Depends on",
                    },
                    {
                        "files": [
                            {
                                "filename": "utf-16BE.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "txts/utf-16BE.txt",
                                "title": "UTF-16BE",
                            }
                        ],
                        "type": "Required by",
                    },
                    {"files": [], "type": "Verified with"},
                ],
                "dependsOn": ["txts/utf-16BE.txt"],
                "requiredBy": ["txts/utf-16BE.txt"],
                "title": "UTF-16LE",
            },
            {
                "title": "UTF-16LE",
            },
        ),
    ]

    def front_matter_data(path: str) -> dict[str, Any]:
        def force_read():
            try:
                return (targets / path).read_text(encoding="utf-8")
            except UnicodeDecodeError:
                return ""

        return {
            "attributes": {},
            "dependencies": [
                {"files": [], "type": "Depends on"},
                {"files": [], "type": "Required by"},
                {"files": [], "type": "Verified with"},
            ],
            "dependsOn": [],
            "requiredBy": [],
            "embedded": [
                {
                    "code": force_read(),
                    "name": "default",
                }
            ],
            "isFailed": False,
            "isVerificationFile": False,
            "path": path,
            "pathExtension": "txt",
            "timestamp": dummy_commit_time([pathlib.Path(path)]).isoformat(sep=" "),
            "verificationStatus": "LIBRARY_NO_TESTS",
            "verifiedWith": [],
        }

    markdawns = [
        MarkdownData(
            path=path,
            front_matter={
                "documentation_of": path,
                "layout": "document",
                "data": front_matter_data(path) | data,
            }
            | root,
            content=content,
        )
        for path, data, root, content in text_files
    ]

    for t in markdawns:
        with subtests.test(msg=t.path):  # pyright: ignore[reportUnknownMemberType]
            front_matter, content = split_front_matter_raw(
                (destination / f"{t.path}.md").read_bytes()
            )
            assert content == t.content
            assert front_matter
            assert yaml.safe_load(front_matter) == t.front_matter

    with subtests.test(  # pyright: ignore[reportUnknownMemberType]
        msg="display/multi.md"
    ):
        front_matter, content = split_front_matter_raw(
            (destination / "display/multi.md").read_bytes()
        )
        assert content == b""
        assert front_matter
        assert yaml.safe_load(front_matter) == {
            "data": {
                "codes": [
                    {
                        "attributes": {},
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "multi-no-keep.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/multi-no-keep.txt",
                                        "title": "Multi no keep $1$",
                                    },
                                    {
                                        "filename": "no-index.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/no-index.txt",
                                        "title": "display=no-index",
                                    },
                                ],
                                "type": "Depends on",
                            },
                            {
                                "files": [
                                    {
                                        "filename": "no-index.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/no-index.txt",
                                        "title": "display=no-index",
                                    }
                                ],
                                "type": "Required by",
                            },
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [
                            "display/multi-no-keep.txt",
                            "display/no-index.txt",
                        ],
                        "documentContent": "# Visible",
                        "documentPath": "display/visible.md",
                        "embedded": [{"code": "", "name": "default"}],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "display/visible.txt",
                        "pathExtension": "txt",
                        "requiredBy": ["display/no-index.txt"],
                        "timestamp": "2029-10-02 13:51:10.570000-05:00",
                        "title": "display=visible",
                        "verificationStatus": "LIBRARY_NO_TESTS",
                        "verifiedWith": [],
                    },
                    {
                        "attributes": {},
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "multi-no-keep.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/multi-no-keep.txt",
                                        "title": "Multi no keep $1$",
                                    },
                                    {
                                        "filename": "visible.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/visible.txt",
                                        "title": "display=visible",
                                    },
                                ],
                                "type": "Depends on",
                            },
                            {
                                "files": [
                                    {
                                        "filename": "visible.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/visible.txt",
                                        "title": "display=visible",
                                    }
                                ],
                                "type": "Required by",
                            },
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [
                            "display/multi-no-keep.txt",
                            "display/visible.txt",
                        ],
                        "documentContent": "",
                        "documentPath": "display/no-index.md",
                        "embedded": [{"code": "", "name": "default"}],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "display/no-index.txt",
                        "pathExtension": "txt",
                        "requiredBy": ["display/visible.txt"],
                        "timestamp": "2029-10-02 13:51:10.570000-05:00",
                        "title": "display=no-index",
                        "verificationStatus": "LIBRARY_NO_TESTS",
                        "verifiedWith": [],
                    },
                    {
                        "attributes": {},
                        "dependencies": [
                            {
                                "files": [
                                    {
                                        "filename": "multi-no-keep.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/multi-no-keep.txt",
                                        "title": "Multi no keep $1$",
                                    },
                                    {
                                        "filename": "no-index.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/no-index.txt",
                                        "title": "display=no-index",
                                    },
                                    {
                                        "filename": "visible.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/visible.txt",
                                        "title": "display=visible",
                                    },
                                ],
                                "type": "Depends on",
                            },
                            {
                                "files": [
                                    {
                                        "filename": "no-index.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/no-index.txt",
                                        "title": "display=no-index",
                                    },
                                    {
                                        "filename": "visible.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/visible.txt",
                                        "title": "display=visible",
                                    },
                                ],
                                "type": "Required by",
                            },
                            {"files": [], "type": "Verified with"},
                        ],
                        "dependsOn": [
                            "display/multi-no-keep.txt",
                            "display/no-index.txt",
                            "display/visible.txt",
                        ],
                        "documentContent": "",
                        "documentPath": "display/hidden.md",
                        "embedded": [{"code": "", "name": "default"}],
                        "isFailed": False,
                        "isVerificationFile": False,
                        "path": "display/hidden.txt",
                        "pathExtension": "txt",
                        "requiredBy": ["display/no-index.txt", "display/visible.txt"],
                        "timestamp": "2029-10-02 13:51:10.570000-05:00",
                        "title": "display=hidden",
                        "verificationStatus": "LIBRARY_NO_TESTS",
                        "verifiedWith": [],
                    },
                ],
                "dependencies": [
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep.txt",
                                "title": "Multi no keep $1$",
                            },
                        ],
                        "type": "Depends on",
                    },
                    {
                        "files": [],
                        "type": "Required by",
                    },
                    {"files": [], "type": "Verified with"},
                ],
                "isFailed": False,
                "path": "display/multi.md",
                "verificationStatus": "LIBRARY_NO_TESTS",
            },
            "documentation_of": ["visible.txt", "no-index.txt", "hidden.txt"],
            "keep_single": True,
            "layout": "multidoc",
            "title": "Multi",
        }

    with subtests.test(  # pyright: ignore[reportUnknownMemberType]
        msg="display/multi-no-keep.txt"
    ):
        front_matter, content = split_front_matter_raw(
            (destination / "display/multi-no-keep.txt.md").read_bytes()
        )
        assert content == b""
        assert front_matter
        assert yaml.safe_load(front_matter) == {
            "data": {
                "attributes": {"TITLE": "Multi no keep $1$"},
                "dependencies": [
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep2.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep2.txt",
                                "title": "Multi no keep $2$",
                            },
                        ],
                        "type": "Depends on",
                    },
                    {
                        "files": [
                            {
                                "filename": "multi-no-keep2.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/multi-no-keep2.txt",
                                "title": "Multi no keep $2$",
                            },
                            {
                                "filename": "no-index.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/no-index.txt",
                                "title": "display=no-index",
                            },
                            {
                                "filename": "visible.txt",
                                "icon": "LIBRARY_NO_TESTS",
                                "path": "display/visible.txt",
                                "title": "display=visible",
                            },
                        ],
                        "type": "Required by",
                    },
                    {"files": [], "type": "Verified with"},
                ],
                "dependsOn": ["display/multi-no-keep2.txt"],
                "embedded": [{"code": "no-keep", "name": "default"}],
                "isFailed": False,
                "isVerificationFile": False,
                "path": "display/multi-no-keep.txt",
                "pathExtension": "txt",
                "requiredBy": [
                    "display/multi-no-keep2.txt",
                    "display/no-index.txt",
                    "display/visible.txt",
                ],
                "timestamp": "2017-12-03 09:56:02.050000-07:00",
                "verificationStatus": "LIBRARY_NO_TESTS",
                "verifiedWith": [],
                "title": "Multi no keep $1$",
            },
            "display": "no-index",
            "documentation_of": "display/multi-no-keep.txt",
            "layout": "document",
            "redirect_to": "/display/multi-no-keep",
            "title": "Multi no keep $1$",
        }

    def check_index_md():
        front_matter, content = split_front_matter_raw(
            (destination / "index.md").read_bytes()
        )
        assert front_matter
        assert yaml.safe_load(front_matter) == {
            "data": {
                "top": [
                    {
                        "categories": [
                            {
                                "name": "",
                                "pages": [
                                    {
                                        "filename": "root.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "root.txt",
                                    }
                                ],
                            },
                            {
                                "name": "a/b/",
                                "pages": [
                                    {
                                        "filename": "c.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "a/b/c.txt",
                                    }
                                ],
                            },
                            {
                                "name": "consolidate/",
                                "pages": [
                                    {
                                        "filename": "1.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/1.txt",
                                    },
                                    {
                                        "filename": "2.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/2.txt",
                                    },
                                    {
                                        "filename": "z/1.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/z/1.txt",
                                    },
                                    {
                                        "filename": "z/2.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/z/2.txt",
                                    },
                                    {
                                        "filename": "z/b/1.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/z/b/1.txt",
                                    },
                                    {
                                        "filename": "z/b/2.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/z/b/2.txt",
                                    },
                                ],
                            },
                            {
                                "name": "consolidate/a/",
                                "pages": [
                                    {
                                        "filename": "1.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/a/1.txt",
                                    },
                                    {
                                        "filename": "2.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/a/2.txt",
                                    },
                                    {
                                        "filename": "b/1.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/a/b/1.txt",
                                    },
                                    {
                                        "filename": "b/2.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "consolidate/a/b/2.txt",
                                    },
                                ],
                            },
                            {
                                "name": "display/",
                                "pages": [
                                    {
                                        "filename": "multi.md",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/multi",
                                        "title": "Multi",
                                    },
                                    {
                                        "filename": "multi-no-keep.md",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/multi-no-keep",
                                        "title": "Multi no keep",
                                    },
                                    {
                                        "filename": "visible.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "display/visible.txt",
                                        "title": "display=visible",
                                    },
                                ],
                            },
                            {
                                "name": "txts/",
                                "pages": [
                                    {
                                        "filename": "utf-16BE.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "txts/utf-16BE.txt",
                                        "title": "UTF-16BE",
                                    },
                                    {
                                        "filename": "utf-16LE.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "txts/utf-16LE.txt",
                                        "title": "UTF-16LE",
                                    },
                                    {
                                        "filename": "utf-8👍.txt",
                                        "icon": "LIBRARY_NO_TESTS",
                                        "path": "txts/utf-8👍.txt",
                                    },
                                ],
                            },
                        ],
                        "type": "Library Files",
                    },
                    {"categories": [], "type": "Verification Files"},
                ]
            },
            "layout": "toppage",
        }
        assert content == b""

    check_index_md()



import datetime
import pathlib
from logging import getLogger
from typing import TYPE_CHECKING, Any, Optional

from onlinejudge_command.subcommand.test import JudgeStatus
from pydantic import BaseModel, Field, field_validator

from competitive_verifier.util import to_relative

from .path import ForcePosixPath
from .result_status import ResultStatus

if TYPE_CHECKING:
    from _typeshed import StrPath

logger = getLogger(__name__)


class TestcaseResult(BaseModel):
    name: str = Field(
        description="The name of test case.",
    )
    """The name of test case.
    """

    status: JudgeStatus = Field(
        description="The result status of the test case.",
    )
    """The result status of the test case.
    """

    elapsed: float = Field(
        description="Number of seconds elapsed for the test case.",
    )
    """Number of seconds elapsed for the test case.
    """

    memory: Optional[float] = Field(
        default=None,
        description="The size of memory used in megabytes.",
    )
    """The size of memory used in megabytes.
    """


class VerificationResult(BaseModel):
    verification_name: Optional[str] = Field(
        default=None,
        description="The name of verification.",
    )
    """The name of verification.
    """
    status: ResultStatus = Field(
        description="The result status of verification.",
    )
    """The result status of verification.
    """

    elapsed: float = Field(
        description="Total number of seconds elapsed for all test cases.",
    )
    """Total number of seconds elapsed for all test cases.
    """

    slowest: Optional[float] = Field(
        default=None,
        description="Maximum number of seconds elapsed for each test cases.",
    )
    """Maximum number of seconds elapsed for each test cases.
    """

    heaviest: Optional[float] = Field(
        default=None,
        description="Maximum size of memory used in megabytes.",
    )
    """Maximum size of memory used in megabytes.
    """

    testcases: Optional[list[TestcaseResult]] = Field(
        default=None,
        description="The results of each test case.",
    )
    """The results of each test case.
    """

    last_execution_time: datetime.datetime = Field(
        default_factory=lambda: datetime.datetime.now(datetime.timezone.utc),
        description="The time at which the last validation was performed.",
    )
    """The time at which the last validation was performed.
    """

    @field_validator("status", mode="before")
    @classmethod
    def verification_list(cls, v: Any) -> Any:
        if isinstance(v, str):
            return v.lower()
        return v

    def need_reverifying(self, base_time: datetime.datetime) -> bool:
        if self.status != ResultStatus.SUCCESS:
            return True

        return self.last_execution_time < base_time


class FileResult(BaseModel):
    verifications: list[VerificationResult] = Field(
        default_factory=list,
        description="The results of each verification.",
    )
    """The results of each verification.
    """

    newest: bool = Field(
        default=True,
        description="Whether the verification was performed on the most recent run.",
    )
    """Whether the verification was performed on the most recent run.
    """

    def need_verification(self, base_time: datetime.datetime) -> bool:
        if len(self.verifications) == 0:
            return True
        return any(r.need_reverifying(base_time) for r in self.verifications)

    def is_success(self, allow_skip: bool) -> bool:
        if allow_skip:
            return all(r.status != ResultStatus.FAILURE for r in self.verifications)
        else:
            return all(r.status == ResultStatus.SUCCESS for r in self.verifications)


class VerifyCommandResult(BaseModel):
    total_seconds: float = Field(
        description="Total number of seconds elapsed for all verification.",
    )
    """Total number of seconds elapsed for all verification.
    """

    files: dict[ForcePosixPath, FileResult] = Field(
        default_factory=dict,
        description="The files to be verified.",
    )
    """The files to be verified.
    """

    @classmethod
    def parse_file_relative(
        cls, path: "StrPath", **kwargs: Any
    ) -> "VerifyCommandResult":
        with pathlib.Path(path).open("rb") as p:
            impl = cls.model_validate_json(p.read(), **kwargs)
        new_files: dict[pathlib.Path, FileResult] = {}
        for p, f in impl.files.items():
            p = to_relative(p)
            if not p:
                continue
            new_files[p] = f

        impl.files = new_files
        return impl

    def merge(self, other: "VerifyCommandResult") -> "VerifyCommandResult":
        d = self.files.copy()
        for k, r in other.files.items():
            cur = d.get(k)
            if r.newest or (cur is None) or (not cur.newest):
                d[k] = r
        return VerifyCommandResult(
            total_seconds=self.total_seconds + other.total_seconds,
            files=d,
        )

    def is_success(self, allow_skip: bool = True) -> bool:
        return all(f.is_success(allow_skip) for f in self.files.values())



import os
import pathlib
import shutil
import subprocess

import pytest

import competitive_verifier.oj as oj

from .types import ConfigDirSetter


@pytest.mark.usefixtures("additional_path")
@pytest.mark.order(-1)
class TestCommandDownload:
    @pytest.mark.integration
    @pytest.mark.skipif(
        not bool(shutil.which("g++") or os.getenv("CXX")),
        reason="g++ is not found",
    )
    def test_library_checker(self, set_config_dir: ConfigDirSetter):
        url = "https://judge.yosupo.jp/problem/aplusb"
        dst_path = set_config_dir("download/library_checker")
        promlem_path = oj.get_directory(url)
        assert dst_path.absolute().is_relative_to(pathlib.Path.cwd())
        assert promlem_path.is_relative_to(dst_path)
        oj.download(url)
        checker = promlem_path / "checker"
        outputs = list(promlem_path.glob("test/*.out"))
        assert outputs
        for out_path in outputs:
            in_path = out_path.with_suffix(".out")
            assert in_path.exists()
            assert (
                subprocess.run(
                    [
                        checker.as_posix(),
                        in_path.as_posix(),
                        out_path.as_posix(),
                        out_path.as_posix(),
                    ]
                ).returncode
                == 0
            )

    @pytest.mark.integration
    @pytest.mark.skipif(
        not bool(os.getenv("YUKICODER_TOKEN")),
        reason="$YUKICODER_TOKEN is not found",
    )
    def test_yukicoder(self, set_config_dir: ConfigDirSetter):
        url = "https://yukicoder.me/problems/no/3040"
        dst_path = set_config_dir("download/yukicoder")

        promlem_path = oj.get_directory(url)
        assert dst_path.absolute().is_relative_to(pathlib.Path.cwd())
        assert promlem_path.is_relative_to(dst_path)

        oj.download(url)

        outputs = list(promlem_path.glob("test/*.out"))
        assert outputs
        for out_path in outputs:
            in_path = out_path.with_suffix(".out")
            assert in_path.exists()
            assert out_path.read_bytes().strip() == b"A + B"

    @pytest.mark.integration
    def test_aizu_onlinejudge(self, set_config_dir: ConfigDirSetter):
        url = "https://onlinejudge.u-aizu.ac.jp/courses/lesson/2/ITP1/1/ITP1_1_A"
        dst_path = set_config_dir("download/aoj")

        promlem_path = oj.get_directory(url)
        assert dst_path.absolute().is_relative_to(pathlib.Path.cwd())
        assert promlem_path.is_relative_to(dst_path)

        oj.download(url)

        outputs = list(promlem_path.glob("test/*.out"))
        assert outputs
        for out_path in outputs:
            in_path = out_path.with_suffix(".out")
            assert in_path.exists()
            assert out_path.read_bytes().strip() == b"Hello World"



import pathlib
from os import PathLike
from typing import Any, Optional

from charset_normalizer import from_path  # pyright: ignore[reportUnknownVariableType]
from charset_normalizer import from_bytes


def to_relative(path: pathlib.Path) -> Optional[pathlib.Path]:
    try:
        return path.resolve().relative_to(pathlib.Path.cwd())
    except ValueError:
        return None


def read_text_normalized(path: PathLike[Any]) -> str:
    return str(from_path(path).best())


def normalize_bytes_text(b: bytes) -> str:
    return str(from_bytes(b).best())



import enum
import pathlib
from functools import cached_property
from logging import getLogger
from typing import TYPE_CHECKING, Any, NamedTuple, Optional, Union

from pydantic import BaseModel, Field

from competitive_verifier.util import to_relative

from ._scc import SccGraph
from .path import ForcePosixPath, SortedPathSet
from .result import FileResult
from .verification import Verification

if TYPE_CHECKING:
    from _typeshed import StrPath
logger = getLogger(__name__)

_DependencyEdges = dict[pathlib.Path, set[pathlib.Path]]


class _DependencyGraph(NamedTuple):
    depends_on: _DependencyEdges
    required_by: _DependencyEdges
    verified_with: _DependencyEdges


class DocumentOutputMode(str, enum.Enum):
    visible = "visible"
    """The document will be output. (default)
    """

    hidden = "hidden"
    """The document will be output but will not linked from other pages.
    """

    no_index = "no-index"
    """The document will be output but will not linked from index page.
    """

    never = "never"
    """The document will be never output.
    """


class AddtionalSource(BaseModel):
    name: str = Field(
        examples=["source_name"],
        description="The name of source file.",
    )
    """The name of source file.
    """
    path: ForcePosixPath = Field(
        description="The path source file.",
        examples=["relative_path_of_directory/file_name.cpp"],
    )
    """The path source file.
    """


class VerificationFile(BaseModel):
    dependencies: SortedPathSet = Field(
        default_factory=set,
        description="The list of dependent files as paths relative to root.",
    )
    """The list of dependent files as paths relative to root.
    """
    verification: Union[list[Verification], Verification, None] = Field(
        default_factory=list
    )
    document_attributes: dict[str, Any] = Field(
        default_factory=dict,
        description="The attributes for documentation.",
    )
    """The attributes for documentation.
    """
    additonal_sources: list[AddtionalSource] = Field(
        default_factory=list,
        description="The addtional source paths.",
        examples=[
            [
                AddtionalSource(
                    name="source_name",
                    path=pathlib.Path("relative_path_of_directory/file_name.cpp"),
                ),
            ],
        ],
    )
    """The addtional source paths
    """

    @property
    def title(self) -> Optional[str]:
        """The document title specified as a attributes"""
        title = self.document_attributes.get("TITLE")
        if not title:
            title = self.document_attributes.get("document_title")
        return title

    @property
    def display(self) -> Optional[DocumentOutputMode]:
        """The document output mode as a attributes"""

        d = self.document_attributes.get("DISPLAY")
        if not isinstance(d, str):
            return None
        try:
            return DocumentOutputMode[d.lower().replace("-", "_")]
        except KeyError:
            return None

    @property
    def verification_list(self) -> list[Verification]:
        if self.verification is None:
            return []
        elif isinstance(self.verification, list):
            return self.verification
        return [self.verification]

    def is_verification(self) -> bool:
        return bool(self.verification)

    def is_skippable_verification(self) -> bool:
        """If the effort required for verification is small, treat it as skippable."""
        return self.is_verification() and all(
            v.is_skippable for v in self.verification_list
        )


class VerificationInput(BaseModel):
    files: dict[ForcePosixPath, VerificationFile] = Field(
        default_factory=dict,
        description="The key is relative path from the root.",
    )

    def merge(self, other: "VerificationInput") -> "VerificationInput":
        return VerificationInput(files=self.files | other.files)

    @staticmethod
    def parse_file_relative(path: "StrPath", **kwargs: Any) -> "VerificationInput":
        with pathlib.Path(path).open("rb") as p:
            impl = VerificationInput.model_validate_json(p.read(), **kwargs)
        new_files: dict[pathlib.Path, VerificationFile] = {}
        for p, f in impl.files.items():
            p = to_relative(p)
            if not p:
                continue
            f.dependencies = set(d for d in map(to_relative, f.dependencies) if d)
            new_files[p] = f

        impl.files = new_files
        return impl

    def scc(self, *, reversed: bool = False) -> list[set[pathlib.Path]]:
        """Strongly Connected Component

        Args:
            reversed bool: if True, libraries are ahead. otherwise, tests are ahead.

        Returns:
            list[set[pathlib.Path]]: Strongly Connected Component result
        """
        paths = list(p for p in self.files.keys())
        vers_rev = {v: i for i, v in enumerate(paths)}
        g = SccGraph(len(paths))
        for p, file in self.files.items():
            for e in file.dependencies:
                t = vers_rev.get(e, -1)
                if 0 <= t:
                    if reversed:
                        g.add_edge(t, vers_rev[p])
                    else:
                        g.add_edge(vers_rev[p], t)
        return [set(paths[ix] for ix in ls) for ls in g.scc()]

    @cached_property
    def transitive_depends_on(self) -> _DependencyEdges:
        d: _DependencyEdges = {}
        g = self.scc(reversed=True)
        for group in g:
            result = group.copy()
            for p in group:
                for dep in self.files[p].dependencies:
                    if dep not in result:
                        resolved = d.get(dep)
                        if resolved is not None:
                            result.update(resolved)
            for p in group:
                d[p] = result

        return d

    @cached_property
    def _dependency_graph(
        self,
    ) -> _DependencyGraph:
        """
        Returns: Dependency graphs
        """
        depends_on: _DependencyEdges = {}
        required_by: _DependencyEdges = {}
        verified_with: _DependencyEdges = {}

        # initialize
        for path in self.files.keys():
            depends_on[path] = set()
            required_by[path] = set()
            verified_with[path] = set()

        # build the graph
        for src, vf in self.files.items():
            for dst in vf.dependencies:
                if src == dst:
                    continue
                if dst not in depends_on:
                    msg = (
                        "The file `%s` which is depended from `%s` is ignored "
                        "because it's not listed as a source code file."
                    )
                    logger.warning(msg, dst, src)
                    continue

                depends_on[src].add(dst)
                if vf.is_verification():
                    verified_with[dst].add(src)
                else:
                    required_by[dst].add(src)
        return _DependencyGraph(
            depends_on=depends_on,
            required_by=required_by,
            verified_with=verified_with,
        )

    @property
    def depends_on(self) -> _DependencyEdges:
        return self._dependency_graph.depends_on

    @property
    def required_by(self) -> _DependencyEdges:
        return self._dependency_graph.required_by

    @property
    def verified_with(self) -> _DependencyEdges:
        return self._dependency_graph.verified_with

    def filterd_files(self, files: dict[ForcePosixPath, "FileResult"]):
        for k, v in files.items():
            if k in self.files:
                yield k, v



import pathlib
from abc import ABC
from typing import Any, Optional

import pytest

from ..types import ConfigDirSetter, FilePaths


class IntegrationData(ABC):
    config_dir_path: pathlib.Path
    targets_path: pathlib.Path
    file_paths: FilePaths

    def __init__(
        self,
        monkeypatch: pytest.MonkeyPatch,
        set_config_dir: ConfigDirSetter,
        file_paths: FilePaths,
    ) -> None:
        self.config_dir_path = set_config_dir(self.config_dir_name)
        self.file_paths = file_paths
        self.targets_path = file_paths.root / self.input_name
        monkeypatch.chdir(self.targets_path)

    @classmethod
    @property
    def input_name(cls) -> str:
        return cls.__name__

    @classmethod
    @property
    def config_dir_name(cls) -> str:
        return cls.__name__

    @property
    def config_path(self) -> Optional[str]:
        ...

    @property
    def include_path(self) -> Optional[list[str]]:
        ...

    @property
    def exclude_path(self) -> Optional[list[str]]:
        ...

    def assert_extra(self):
        pass

    def check_envinronment(self) -> bool:
        return True

    def expected_verify_json(self) -> dict[str, Any]:
        ...

    def expected_verify_result(self) -> dict[str, Any]:
        ...



from .tools.download_command import run_wrapper as download
from .tools.func import get_directory
from .tools.test_command import check_gnu_time
from .tools.test_command import run_wrapper as test

__all__ = [
    "check_gnu_time",
    "get_directory",
    "download",
    "test",
]



"""https://github.com/smoofra/mslex/blob/master/mslex.py

"""

import re
import shlex
import sys

cmd_meta: str = r"([\"\^\&\|\<\>\(\)\%\!])"
cmd_meta_or_space: str = r"[\s\"\^\&\|\<\>\(\)\%\!]"

cmd_meta_inside_quotes: str = r"([\"\%\!])"


def _msquote(s: str, *, for_cmd: bool = True) -> str:
    """
    Quote a string for use as a command line argument in DOS or Windows.

    On windows, before a command line argument becomes a char* in a
    program's argv, it must be parsed by both cmd.exe, and by
    CommandLineToArgvW.

    If for_cmd is true, then this will quote the string so it will
    be parsed correctly by cmd.exe and then by CommandLineToArgvW.

    If for_cmd is false, then this will quote the string so it will
    be parsed correctly when passed directly to CommandLineToArgvW.

    For some strings there is no way to quote them so they will
    parse correctly in both situations.
    """
    if not s:
        return '""'
    if not re.search(cmd_meta_or_space, s):
        return s
    if for_cmd and re.search(cmd_meta, s):
        if not re.search(cmd_meta_inside_quotes, s):
            m = re.search(r"\\+$", s)
            if m:
                return '"' + s + m.group() + '"'
            else:
                return '"' + s + '"'
        if not re.search(r"[\s\"]", s):
            return re.sub(cmd_meta, r"^\1", s)
        return re.sub(cmd_meta, r"^\1", _msquote(s, for_cmd=False))
    i = re.finditer(r"(\\*)(\"+)|(\\+)|([^\\\"]+)", s)

    def parts():
        yield '"'
        for m in i:
            _, end = m.span()
            slashes, quotes, onlyslashes, text = m.groups()
            if quotes:
                yield slashes
                yield slashes
                yield r"\"" * len(quotes)
            elif onlyslashes:
                if end == len(s):
                    yield onlyslashes
                    yield onlyslashes
                else:
                    yield onlyslashes
            else:
                yield text
        yield '"'

    return "".join(parts())


quote = _msquote if sys.platform == "win32" else shlex.quote
split = shlex.split


def join(args: list[str]) -> str:
    """shlex or smoofra/mslex"""
    if sys.platform == "win32":
        return " ".join(_msquote(arg) for arg in args)
    return shlex.join(args)



import datetime
import enum
from typing import Annotated, Any, Optional, Sequence

from pydantic import BaseModel, ConfigDict, PlainSerializer, model_validator
from pydantic.alias_generators import to_camel

from competitive_verifier.models import ForcePosixPath, SortedPathList, TestcaseResult


class StatusIcon(str, enum.Enum):
    @property
    def is_failed(self) -> bool:
        return not self.is_success

    @property
    def is_success(self) -> bool:
        return (
            self == self.LIBRARY_ALL_AC
            or self == self.LIBRARY_PARTIAL_AC
            or self == self.LIBRARY_NO_TESTS
            or self == self.TEST_ACCEPTED
            or self == self.TEST_WAITING_JUDGE
        )

    @property
    def is_test(self) -> bool:
        return not self.is_library

    @property
    def is_library(self) -> bool:
        return (
            self == self.LIBRARY_ALL_AC
            or self == self.LIBRARY_PARTIAL_AC
            or self == self.LIBRARY_SOME_WA
            or self == self.LIBRARY_ALL_WA
            or self == self.LIBRARY_NO_TESTS
        )

    LIBRARY_ALL_AC = "LIBRARY_ALL_AC"
    LIBRARY_PARTIAL_AC = "LIBRARY_PARTIAL_AC"
    LIBRARY_SOME_WA = "LIBRARY_SOME_WA"
    LIBRARY_ALL_WA = "LIBRARY_ALL_WA"
    LIBRARY_NO_TESTS = "LIBRARY_NO_TESTS"
    TEST_ACCEPTED = "TEST_ACCEPTED"
    TEST_WRONG_ANSWER = "TEST_WRONG_ANSWER"
    TEST_WAITING_JUDGE = "TEST_WAITING_JUDGE"


class RenderBaseModel(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        extra="allow",
    )


class RenderLink(RenderBaseModel):
    path: ForcePosixPath
    filename: str
    icon: StatusIcon
    title: Optional[str] = None

    @model_validator(mode="after")
    def validate_title(self) -> "RenderLink":
        if self.path.as_posix() == self.title:
            self.title = None
        return self


class Dependency(RenderBaseModel):
    type: str
    files: list[RenderLink]


class EmbeddedCode(RenderBaseModel):
    name: str
    code: str


class EnvTestcaseResult(RenderBaseModel, TestcaseResult):
    environment: Optional[str]


class CategorizedIndex(RenderBaseModel):
    name: str
    pages: list[RenderLink]


class IndexFiles(RenderBaseModel):
    type: str
    categories: list[CategorizedIndex]


class PageRenderData(RenderBaseModel):
    path: ForcePosixPath
    path_extension: str
    title: Optional[str]
    embedded: list[EmbeddedCode]

    timestamp: Annotated[
        datetime.datetime,
        PlainSerializer(lambda x: str(x), return_type=str, when_used="json"),
    ]
    attributes: dict[str, Any]
    testcases: Optional[list[EnvTestcaseResult]] = None

    is_failed: bool
    is_verification_file: bool
    verification_status: StatusIcon

    depends_on: SortedPathList
    required_by: SortedPathList
    verified_with: SortedPathList

    document_path: Optional[ForcePosixPath] = None
    dependencies: list[Dependency]


class CodePageData(PageRenderData):
    document_content: Optional[str]


class MultiCodePageData(RenderBaseModel):
    path: ForcePosixPath
    verification_status: StatusIcon
    is_failed: bool
    codes: Sequence[CodePageData]
    dependencies: list[Dependency]


class IndexRenderData(RenderBaseModel):
    top: list[IndexFiles]



import pathlib
from logging import getLogger
from typing import Any, Optional, Sequence

from pydantic import ValidationInfo, field_validator

import competitive_verifier.oj.verify.shlex2 as shlex
from competitive_verifier.oj.verify.languages.user_defined import UserDefinedLanguage
from competitive_verifier.oj.verify.models import (
    LanguageEnvironment,
    OjVerifyUserDefinedConfig,
)

logger = getLogger(__name__)


class OjVerifyJavaConfig(OjVerifyUserDefinedConfig):
    execute: None = None  # pyright: ignore[reportIncompatibleVariableOverride]

    @field_validator("execute", "compile", mode="before")
    @classmethod
    def name_must_contain_space(cls, v: Any, info: ValidationInfo) -> None:
        if v is None:
            return None
        raise ValueError(f'You cannot overwrite "{info.field_name}" for Java language')


class JavaLanguageEnvironment(LanguageEnvironment):
    @property
    def name(self) -> str:
        return "Java"

    def get_compile_command(
        self, path: pathlib.Path, *, basedir: pathlib.Path, tempdir: pathlib.Path
    ) -> str:
        return shlex.join(["javac", str(basedir / path)])

    def get_execute_command(
        self, path: pathlib.Path, *, basedir: pathlib.Path, tempdir: pathlib.Path
    ) -> str:
        relative_path = (basedir / path).relative_to(basedir)
        class_path = ".".join([*relative_path.parent.parts, relative_path.stem])
        return shlex.join(["java", class_path])


class JavaLanguage(UserDefinedLanguage):
    def __init__(self, *, config: Optional[OjVerifyJavaConfig]):
        super().__init__(extension="java", config=config or OjVerifyJavaConfig())

    def list_environments(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> Sequence[LanguageEnvironment]:
        return [JavaLanguageEnvironment()]



from typing import Optional

from pydantic import Field

from competitive_verifier.models import ShellCommand, ShellCommandLike
from competitive_verifier.oj.verify.languages.user_defined import UserDefinedLanguage
from competitive_verifier.oj.verify.models import OjVerifyUserDefinedConfig


class OjVerifyRubyConfig(OjVerifyUserDefinedConfig):
    execute: ShellCommandLike = Field(
        default_factory=lambda: ShellCommand(command=["ruby", "{basedir}/{path}"]),
    )


class RubyLanguage(UserDefinedLanguage):
    def __init__(self, *, config: Optional[OjVerifyRubyConfig]):
        super().__init__(extension="rb", config=config or OjVerifyRubyConfig())



from typing import TypeVar

from pydantic import BaseModel

T = TypeVar("T")


class SplitState(BaseModel):
    size: int
    index: int

    def __str__(self) -> str:
        return f"{self.index}/{self.size}"

    def split(self, lst: list[T]) -> list[T]:
        """Split list


        Args:
            lst (list[T]): Target list

        Returns:
            list[T]: Splited list

        Example:
            state = SplitState(size=3, index=0)
            assert state.split([0, 1, 2, 3, 4]) == [0]
            state = SplitState(size=3, index=1)
            assert state.split([0, 1, 2, 3, 4]) == [1, 2]
            state = SplitState(size=3, index=2)
            assert state.split([0, 1, 2, 3, 4]) == [3, 4]
            state = SplitState(size=6, index=4)
            assert state.split([0, 1, 2, 3, 4]) == [4]
            state = SplitState(size=6, index=5)
            assert state.split([0, 1, 2, 3, 4]) == []
        """

        if len(lst) <= self.size:
            if len(lst) <= self.index:
                return []
            else:
                return [lst[self.index]]

        from_index = len(lst) * self.index // self.size
        to_index = len(lst) * (self.index + 1) // self.size
        return lst[from_index:to_index]



import importlib.resources
import pathlib
import shutil
from logging import getLogger
from typing import Optional

from pydantic import BaseModel

import competitive_verifier.git as git
import competitive_verifier.github as github
from competitive_verifier.models import VerificationInput, VerifyCommandResult

from .config import ConfigYaml, load_config_yml
from .front_matter import Markdown
from .render import RenderJob
from .static_files import default_resource_files

logger = getLogger(__name__)


_RESOURCE_PACKAGE = "competitive_verifier_resources"
_DOC_USAGE_PATH = "doc_usage.txt"


class DocumentBuilder(BaseModel):
    input: VerificationInput
    result: VerifyCommandResult
    docs_dir: pathlib.Path
    destination_dir: pathlib.Path
    include: Optional[list[str]]
    exclude: Optional[list[str]]

    def build(self) -> bool:
        logger.info(
            "Working directory: %s",
            pathlib.Path.cwd().as_posix(),
        )

        logger.info("Generate documents...")

        # implementation
        result = self.impl()

        logger.info("Generated.")
        logger.info(
            (importlib.resources.files(_RESOURCE_PACKAGE) / _DOC_USAGE_PATH)
            .read_text(encoding="utf-8")
            .replace("{{{{{markdown_dir_path}}}}}", self.destination_dir.as_posix())
            .replace(
                "{{{{{repository}}}}}",
                github.env.get_repository()
                or "competitive-verifier/competitive-verifier",
            )
        )

        return result

    def impl(self) -> bool:
        self.destination_dir.mkdir(parents=True, exist_ok=True)

        config_yml = load_config_yml(self.docs_dir)
        logger.info("_config.yml: %s", config_yml)

        index_md_path = self.docs_dir / "index.md"
        if index_md_path.exists():
            index_md = Markdown.load_file(index_md_path)
        else:
            index_md = None

        # Write code documents.
        self.write_code_docs(
            config_yml=config_yml,
            index_md=index_md,
            static_dir=self.docs_dir / "static",
        )

        # Write _config.yml
        (self.destination_dir / "_config.yml").write_bytes(config_yml.model_dump_yml())

        # Copy static files
        self.copy_static_files(static_dir=self.docs_dir / "static")
        return True

    def copy_static_files(self, *, static_dir: pathlib.Path):
        logger.info("Copy library static files...")
        for path, content in default_resource_files():
            file_dst = self.destination_dir / path
            logger.debug("Writing to %s", file_dst.as_posix())
            if not file_dst.exists():
                file_dst.parent.mkdir(parents=True, exist_ok=True)
            file_dst.write_bytes(content)

        logger.info("Copy user static files...")
        try:
            if static_dir.is_dir():
                shutil.copytree(
                    static_dir,
                    self.destination_dir,
                    dirs_exist_ok=True,
                )
        except Exception as e:
            logger.exception("Failed to copy user static files.", e)

    def write_code_docs(
        self,
        *,
        config_yml: ConfigYaml,
        index_md: Optional[Markdown],
        static_dir: Optional[pathlib.Path],
    ):
        logger.info("Write document files...")

        exclude = (self.exclude or []) + (config_yml.exclude or [])
        if static_dir and static_dir.is_relative_to("."):
            exclude.append(self.docs_dir.relative_to(".").as_posix())

        sources = git.ls_files(*(self.include or []))
        if exclude:
            sources -= git.ls_files(*exclude)

        # yield source.with_suffix(source.suffix + ".md"), source.read_bytes()
        for job in RenderJob.enumerate_jobs(
            sources=sources,
            input=self.input,
            result=self.result,
            config=config_yml,
            index_md=index_md,
        ):
            logger.debug(job)
            dst = self.destination_dir / job.destination_name
            dst.parent.mkdir(parents=True, exist_ok=True)
            logger.info("writing to %s", dst.as_posix())
            with dst.open("wb") as fp:
                job.write_to(fp)



import argparse
import os
import pathlib

COMPETITIVE_VERIFY_FILES_PATH = "COMPETITIVE_VERIFY_FILES_PATH"


def add_verbose_argument(
    parser: argparse.ArgumentParser,
) -> argparse.Action:
    return parser.add_argument(
        "-v", "--verbose", action="store_true", help="Show debug level log."
    )


def add_verify_files_json_argument(
    parser: argparse.ArgumentParser,
    required: bool = True,
) -> argparse.Action:
    default = os.getenv(COMPETITIVE_VERIFY_FILES_PATH)
    return parser.add_argument(
        "--verify-json",
        dest="verify_files_json",
        default=default,
        required=required and not bool(default),
        help="File path of verify_files.json. default: environ variable $COMPETITIVE_VERIFY_FILES_PATH",  # noqa: E501
        type=pathlib.Path,
    )


def add_result_json_argument(parser: argparse.ArgumentParser) -> argparse.Action:
    return parser.add_argument(
        "result_json",
        nargs="+",
        help="Json files which is result of `verify`",
        type=pathlib.Path,
    )


def add_ignore_error_argument(parser: argparse.ArgumentParser) -> argparse.Action:
    return parser.add_argument(
        "--check-error",
        help="Exit not zero if failed verification exists",
        dest="ignore_error",
        action="store_false",
    )


def add_write_summary_argument(parser: argparse.ArgumentParser) -> argparse.Action:
    return parser.add_argument(
        "--write-summary",
        action="store_true",
        help="Write GitHub Actions summary",
    )


def add_include_exclude_argument(
    parser: argparse.ArgumentParser,
) -> tuple[argparse.Action, argparse.Action]:
    return parser.add_argument(
        "--include",
        nargs="*",
        help="Included file",
        default=[],
        type=str,
    ), parser.add_argument(
        "--exclude",
        nargs="*",
        help="Excluded file",
        default=[],
        type=str,
    )



import pathlib
from typing import Any

import pytest
from pytest_mock import MockerFixture

import competitive_verifier.oj as oj
from competitive_verifier.oj.tools.test_command import OjTestArguments

test_oj_test_params: dict[str, tuple[dict[str, Any], OjTestArguments]] = {
    "default": (
        {
            "url": "http://example.com",
            "command": "ls .",
            "tle": 2,
            "error": None,
            "mle": 128,
            "env": None,
        },
        OjTestArguments(
            command="ls .",
            tle=2,
            cookie=pathlib.Path("/bar/baz/online-judge-tools") / "cookie.txt",
            print_input=True,
            mle=128,
            error=None,
            directory=pathlib.Path(
                ".competitive-verifier/cache/problems/a9b9f04336ce0181a08e774e01113b31/test"
            ),
            judge=None,
        ),
    ),
    "with_env": (
        {
            "url": "http://example.com",
            "command": ["ls", "."],
            "tle": 30,
            "error": None,
            "mle": 256,
            "env": {"TOKEN": "Dummy"},
        },
        OjTestArguments(
            command=["ls", "."],
            tle=30,
            cookie=pathlib.Path("/bar/baz/online-judge-tools") / "cookie.txt",
            print_input=True,
            mle=256,
            error=None,
            directory=pathlib.Path(
                ".competitive-verifier/cache/problems/a9b9f04336ce0181a08e774e01113b31/test"
            ),
            judge=None,
            env={"TOKEN": "Dummy"},
        ),
    ),
}


@pytest.mark.parametrize(
    "input, expected",
    test_oj_test_params.values(),
    ids=test_oj_test_params.keys(),
)
def test_oj_test(
    mocker: MockerFixture,
    input: dict[str, Any],
    expected: OjTestArguments,
):
    mocker.patch(
        "competitive_verifier.oj.tools.test_command.get_cache_directory",
        return_value=pathlib.Path("/bar/baz/online-judge-tools"),
    )
    run = mocker.patch("competitive_verifier.oj.tools.test_command.run")

    oj.test(**input)

    run.assert_called_once_with(expected)



from abc import ABC, abstractmethod
from typing import Annotated, Literal, Optional, Protocol, Union

from pydantic import BaseModel, Field

from .result import VerificationResult
from .result_status import ResultStatus
from .shell import ShellCommand, ShellCommandLike


class VerificationParams(Protocol):
    default_tle: Optional[float]
    default_mle: Optional[float]


class BaseVerification(BaseModel, ABC):
    name: Optional[str] = None

    @abstractmethod
    def run(
        self,
        params: Optional[VerificationParams] = None,
    ) -> Union[ResultStatus, VerificationResult]:
        ...

    @abstractmethod
    def run_compile_command(
        self,
        params: Optional[VerificationParams] = None,
    ) -> bool:
        ...

    @property
    def is_skippable(self) -> bool:
        """
        If verification cost is small, it is skippable.
        """
        return False


class ConstVerification(BaseVerification):
    type: Literal["const"] = "const"
    status: ResultStatus = Field(description="The pre-defined result.")
    """The pre-defined result.
    """

    @property
    def is_skippable(self) -> bool:
        return True

    def run(
        self,
        params: Optional[VerificationParams] = None,
    ) -> ResultStatus:
        return self.status

    def run_compile_command(
        self,
        params: Optional[VerificationParams] = None,
    ) -> bool:
        return True


class CommandVerification(BaseVerification):
    type: Literal["command"] = "command"

    command: ShellCommandLike = Field(description="The shell command for verification.")
    """The shell command for verification.
    """
    compile: Optional[ShellCommandLike] = Field(
        default=None,
        description="The shell command for compile.",
    )
    """The shell command for compile.
    """

    def run(
        self,
        params: Optional[VerificationParams] = None,
    ) -> ResultStatus:
        c = ShellCommand.parse_command_like(self.command)
        if c.exec_command(text=True).returncode == 0:
            return ResultStatus.SUCCESS
        return ResultStatus.FAILURE

    def run_compile_command(
        self,
        params: Optional[VerificationParams] = None,
    ) -> bool:
        if self.compile:
            c = ShellCommand.parse_command_like(self.compile)
            return c.exec_command(text=True).returncode == 0
        return True


class ProblemVerification(BaseVerification):
    type: Literal["problem"] = "problem"

    command: ShellCommandLike = Field(description="The shell command for verification.")
    """The shell command for verification.
    """
    compile: Optional[ShellCommandLike] = Field(
        default=None,
        description="The shell command for compile.",
    )
    """The shell command for compile.
    """

    problem: str = Field(
        description="The URL of problem.",
    )
    """
    problem: URL of problem
    """

    error: Optional[float] = Field(
        default=None,
        examples=[1e-9],
        description="The absolute or relative error to be considered as correct.",
    )
    """The absolute or relative error to be considered as correct.
    """
    tle: Optional[float] = Field(
        default=None,
        examples=[10],
        description="The TLE time in seconds.",
    )
    """The TLE time in seconds.
    """
    mle: Optional[float] = Field(
        default=None,
        examples=[64],
        description="The MLE memory size in megabytes.",
    )
    """The MLE memory size in megabytes.
    """

    def run(
        self,
        params: Optional[VerificationParams] = None,
    ) -> VerificationResult:
        import competitive_verifier.oj as oj

        if not params:
            raise ValueError("ProblemVerification.run requires VerificationParams")

        c = ShellCommand.parse_command_like(self.command)
        result = oj.test(
            url=self.problem,
            command=c.command,
            env=c.env,
            tle=self.tle or params.default_tle,
            error=self.error,
            mle=self.mle or params.default_mle,
        )
        result.verification_name = self.name
        return result

    def run_compile_command(
        self,
        params: Optional[VerificationParams] = None,
    ) -> bool:
        if self.compile:
            c = ShellCommand.parse_command_like(self.compile)
            return c.exec_command(text=True).returncode == 0
        return True


Verification = Annotated[
    Union[
        ConstVerification,
        CommandVerification,
        ProblemVerification,
    ],
    Field(discriminator="type"),
]



from typing import Optional

from pydantic import Field

from competitive_verifier.models import ShellCommand, ShellCommandLike
from competitive_verifier.oj.verify.languages.user_defined import UserDefinedLanguage
from competitive_verifier.oj.verify.models import OjVerifyUserDefinedConfig


class OjVerifyHaskellConfig(OjVerifyUserDefinedConfig):
    execute: ShellCommandLike = Field(
        default_factory=lambda: ShellCommand(
            command=["runghc", "{basedir}/{path}"],
        ),
    )


class HaskellLanguage(UserDefinedLanguage):
    def __init__(self, *, config: Optional[OjVerifyHaskellConfig]):
        super().__init__(extension="hs", config=config or OjVerifyHaskellConfig())



# pyright: reportGeneralTypeIssues=false
import io
import textwrap
from typing import Any

import pytest
from pydantic import ValidationError
from pydantic_core import ErrorDetails

from competitive_verifier.oj.verify.list import OjVerifyConfig

default_languages: dict[str, Any] = {
    "cpp": {},
    "go": {
        "execute": {
            "command": ["go", "run", "{basedir}/{path}"],
            "env": {"GO111MODULE": "off"},
        },
    },
    "haskell": {
        "execute": {
            "command": ["runghc", "{basedir}/{path}"],
        },
    },
    "java": {},
    "nim": {"environments": []},
    "ruby": {
        "execute": {
            "command": ["ruby", "{basedir}/{path}"],
        },
    },
    "rust": {},
}


test_oj_resolve_config_load_params: dict[str, tuple[str, dict[str, Any]]] = {
    "not_deifne": (
        textwrap.dedent(""),
        {"languages": default_languages},
    ),
    "empty_deifne": (
        textwrap.dedent(
            """
            [languages.cpp]
            [languages.go]
            [languages.haskell]
            [languages.java]
            [languages.nim]
            [languages.ruby]
            [languages.rust]
            """
        ),
        {"languages": default_languages},
    ),
    "full_define": (
        textwrap.dedent(
            r"""
            [languages.cpp]
            [[languages.cpp.environments]]
            CXX = "g++"
            CXXFLAGS = ["flags"]
            extra = 1
            [[languages.cpp.environments]]
            CXX = "clang++"
            extra = 2
            [languages.go]
            execute = "env GO111MODULE=off go run {basedir}/{path}"
            [languages.haskell]
            execute = "runghc {basedir}/{path}"
            [languages.java]
            [languages.nim]
            [[languages.nim.environments]]
            compile_to = "cpp"
            NIMFLAGS = ["-d:release", "--opt:speed"]
            [languages.ruby]
            execute = "ruby {basedir}/{path}"
            [languages.rust]
            [languages.rust.list_dependencies_backend]
            kind = "cargo-udeps"
            toolchain = "nightly"
            [languages.awk]
            compile = "bash -c 'echo hello > {tempdir}/hello'"
            execute = "env AWKPATH={basedir} awk -f {path}"
            bundle = "false"
            list_dependencies = "sed 's/^@include \"\\(.*\\)\"$/\\1/ ; t ; d' {path}"
            """
        ),
        {
            "languages": {
                "cpp": {
                    "environments": [
                        {"CXX": "g++", "CXXFLAGS": ["flags"]},
                        {"CXX": "clang++"},
                    ]
                },
                "go": {
                    "execute": "env GO111MODULE=off go run {basedir}/{path}",
                },
                "haskell": {
                    "execute": "runghc {basedir}/{path}",
                },
                "java": {},
                "nim": {
                    "environments": [
                        {"compile_to": "cpp", "NIMFLAGS": ["-d:release", "--opt:speed"]}
                    ],
                },
                "ruby": {
                    "execute": "ruby {basedir}/{path}",
                },
                "rust": {
                    "list_dependencies_backend": {
                        "kind": "cargo-udeps",
                        "toolchain": "nightly",
                    }
                },
                "awk": {
                    "compile": "bash -c 'echo hello > {tempdir}/hello'",
                    "execute": "env AWKPATH={basedir} awk -f {path}",
                    "bundle": "false",
                    "list_dependencies": "sed 's/^@include \"\\(.*\\)\"$/\\1/ ; t ; d' {path}",
                },
            },
        },
    ),
    "rust_kind_none": (
        textwrap.dedent(
            """
            [languages.rust.list_dependencies_backend]
            kind = "none"
            """
        ),
        {
            "languages": default_languages
            | {
                "rust": {
                    "list_dependencies_backend": {
                        "kind": "none",
                    }
                },
            }
        },
    ),
}


@pytest.mark.parametrize(
    "toml, expected",
    test_oj_resolve_config_load_params.values(),
    ids=test_oj_resolve_config_load_params.keys(),
)
def test_oj_resolve_config_load(toml: str, expected: Any):
    with io.BytesIO(toml.encode("utf-8")) as fp:
        assert OjVerifyConfig.load(fp).model_dump(exclude_none=True) == expected


test_oj_resolve_config_load_error_params: dict[str, tuple[str, list[ErrorDetails]]] = {
    "cpp_no_CXX": (
        textwrap.dedent(
            r"""
            [languages.cpp]
            [[languages.cpp.environments]]
            CX = "g++"
            """
        ),
        [
            {
                "input": {"CX": "g++"},
                "loc": ("languages", "cpp", "environments", 0, "CXX"),
                "msg": "Field required",
                "type": "missing",
                "url": "https://errors.pydantic.dev/2.4/v/missing",
            },
        ],
    ),
    "java_with_execute_and_compile": (
        textwrap.dedent(
            r"""
            [languages.java]
            compile = "javac {path}"
            execute = "java {path}"
            """
        ),
        [
            {
                "ctx": {
                    "error": ValueError(
                        'You cannot overwrite "execute" for Java language'
                    )
                },
                "input": "java {path}",
                "loc": ("languages", "java", "execute"),
                "msg": 'Value error, You cannot overwrite "execute" for Java language',
                "type": "value_error",
                "url": "https://errors.pydantic.dev/2.4/v/value_error",
            },
            {
                "ctx": {
                    "error": ValueError(
                        'You cannot overwrite "compile" for Java language'
                    )
                },
                "input": "javac {path}",
                "loc": ("languages", "java", "compile"),
                "msg": 'Value error, You cannot overwrite "compile" for Java language',
                "type": "value_error",
                "url": "https://errors.pydantic.dev/2.4/v/value_error",
            },
        ],
    ),
    "user_defined_no_execute": (
        textwrap.dedent(
            r"""
            [languages.awk]
            compile = "bash -c 'echo hello > {tempdir}/hello'"
            bundle = "false"
            list_dependencies = "sed 's/^@include \"\\(.*\\)\"$/\\1/ ; t ; d' {path}"
            """
        ),
        [
            {
                "input": {
                    "bundle": "false",
                    "compile": "bash -c 'echo hello > {tempdir}/hello'",
                    "list_dependencies": "sed 's/^@include \"\\(.*\\)\"$/\\1/ ; t ; d' "
                    "{path}",
                },
                "loc": ("languages", "awk", "execute"),
                "msg": "Field required",
                "type": "missing",
                "url": "https://errors.pydantic.dev/2.4/v/missing",
            },
        ],
    ),
    "user_defined_no_execute_2": (
        textwrap.dedent(
            r"""
            [languages.awk]
            compile = "bash -c 'echo hello > {tempdir}/hello'"
            bundle = "false"
            list_dependencies = "sed 's/^@include \"\\(.*\\)\"$/\\1/ ; t ; d' {path}"
            [languages.txt]
            """
        ),
        [
            {
                "input": {
                    "bundle": "false",
                    "compile": "bash -c 'echo hello > {tempdir}/hello'",
                    "list_dependencies": "sed 's/^@include \"\\(.*\\)\"$/\\1/ ; t ; d' "
                    "{path}",
                },
                "loc": ("languages", "awk", "execute"),
                "msg": "Field required",
                "type": "missing",
                "url": "https://errors.pydantic.dev/2.4/v/missing",
            },
            {
                "input": {},
                "loc": ("languages", "txt", "execute"),
                "msg": "Field required",
                "type": "missing",
                "url": "https://errors.pydantic.dev/2.4/v/missing",
            },
        ],
    ),
}


@pytest.mark.parametrize(
    "toml, expected_error",
    test_oj_resolve_config_load_error_params.values(),
    ids=test_oj_resolve_config_load_error_params.keys(),
)
def test_oj_resolve_config_load_error(toml: str, expected_error: list[ErrorDetails]):
    with io.BytesIO(toml.encode("utf-8")) as fp:
        with pytest.raises(ValidationError) as excinfo:
            OjVerifyConfig.load(fp)
        e: ValidationError = excinfo.value
        errors = e.errors()
        assert len(errors) == len(expected_error)
        for i, ex in enumerate(errors):
            expected_ctx = expected_error[i].get("ctx")
            if expected_ctx and expected_ctx.get("error"):
                assert ex["ctx"]["error"].args == expected_ctx["error"].args
                del ex["ctx"]["error"]
                del expected_ctx["error"]
        assert errors == expected_error



import pathlib

from competitive_verifier.documents.static_files import default_resource_files


def test_jekyll_staic_files():
    resources = list(default_resource_files())
    src_resources_jekyll_root = (
        pathlib.Path(__file__).parent.parent.parent
        / "src/competitive_verifier_resources/jekyll"
    )
    src_resources_path = list(src_resources_jekyll_root.glob("**/*"))
    src_resources = [
        (p.relative_to(src_resources_jekyll_root).as_posix(), p.read_bytes())
        for p in src_resources_path
        if p.is_file()
    ]
    assert sorted(src_resources, key=lambda r: r[0]) == sorted(
        resources, key=lambda r: r[0]
    )



import datetime
import pathlib
from typing import TYPE_CHECKING, Iterable

from .exec import exec_command

if TYPE_CHECKING:
    from _typeshed import StrPath


def get_commit_time(files: Iterable[pathlib.Path]) -> datetime.datetime:
    code = ["git", "log", "-1", "--date=iso", "--pretty=%ad", "--"] + list(
        map(str, files)
    )
    stdout = exec_command(code, text=True, capture_output=True).stdout
    timestamp = stdout.strip()
    if not timestamp:
        return datetime.datetime.min.replace(tzinfo=datetime.timezone.utc)
    return datetime.datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S %z")


def ls_files(*args: "StrPath") -> set[pathlib.Path]:
    stdout = exec_command(
        ["git", "ls-files", "-z"] + list(str(p) for p in (args or [])),
        text=True,
        capture_output=True,
    ).stdout
    return set(map(pathlib.Path, filter(lambda p: p, stdout.split("\0"))))


def get_root_directory() -> pathlib.Path:
    stdout = exec_command(
        ["git", "rev-parse", "--show-toplevel"], text=True, capture_output=True
    ).stdout
    return pathlib.Path(stdout.strip())



import argparse
import logging
import pathlib
import sys
from logging import getLogger
from typing import Optional, Union

from pydantic import ValidationError

from competitive_verifier.arg import add_include_exclude_argument, add_verbose_argument
from competitive_verifier.log import configure_stderr_logging
from competitive_verifier.oj.verify.list import OjVerifyConfig

from .resolver import OjResolver

logger = getLogger(__name__)


def run_impl(
    include: list[str],
    exclude: list[str],
    config: Union[pathlib.Path, OjVerifyConfig, None],
    enable_bundle: bool,
) -> bool:
    if config is None:
        logger.info("no config file")
        config = OjVerifyConfig()
    elif not isinstance(config, OjVerifyConfig):
        try:
            config_path = config
            with pathlib.Path(config_path).open("rb") as fp:
                config = OjVerifyConfig.load(fp)
                logger.info("config file loaded: %s: %s", str(config_path), config)
        except ValidationError as e:
            logger.error("config file validation error: %s", e)
            config = OjVerifyConfig()

    resolver = OjResolver(
        include=include,
        exclude=exclude,
        config=config,
    )
    resolved = resolver.resolve(bundle=enable_bundle)
    print(resolved.model_dump_json(exclude_none=True))
    return True


def run(args: argparse.Namespace) -> bool:
    default_level = logging.INFO
    if args.verbose:
        default_level = logging.DEBUG
    configure_stderr_logging(default_level)

    return run_impl(
        include=args.include,
        exclude=args.exclude,
        config=args.config,
        enable_bundle=args.bundle,
    )


def argument(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    add_verbose_argument(parser)
    add_include_exclude_argument(parser)
    parser.add_argument(
        "--no-bundle",
        dest="bundle",
        action="store_false",
        help="Disable bundle",
    )
    parser.add_argument(
        "--config",
        help="config.toml",
        type=pathlib.Path,
    )

    return parser


def main(args: Optional[list[str]] = None) -> None:
    try:
        parsed = argument(argparse.ArgumentParser()).parse_args(args)
        if not run(parsed):
            sys.exit(1)
    except Exception as e:
        sys.stderr.write(str(e))
        sys.exit(2)


if __name__ == "__main__":
    main()



import os
import pathlib
from logging import getLogger
from typing import Any, Optional, Union

import yaml
from pydantic import BaseModel, ConfigDict, Field

from competitive_verifier import git, github
from competitive_verifier.models import RelativeDirectoryPath, SortedPathSet

_CONFIG_YML_PATH = "_config.yml"

COMPETITIVE_VERIFY_DOCS_CONFIG_YML = "COMPETITIVE_VERIFY_DOCS_CONFIG_YML"

logger = getLogger(__name__)


class ConfigIcons(BaseModel):
    model_config = ConfigDict(extra="allow")

    LIBRARY_ALL_AC: str = ":heavy_check_mark:"
    LIBRARY_PARTIAL_AC: str = ":heavy_check_mark:"
    LIBRARY_SOME_WA: str = ":question:"
    LIBRARY_ALL_WA: str = ":x:"
    LIBRARY_NO_TESTS: str = ":warning:"
    TEST_ACCEPTED: str = ":heavy_check_mark:"
    TEST_WRONG_ANSWER: str = ":x:"
    TEST_WAITING_JUDGE: str = ":warning:"


class ConfigYaml(BaseModel):
    model_config = ConfigDict(extra="allow")

    def merge(self, other: "ConfigYaml"):
        return ConfigYaml.model_validate(
            self.model_dump() | other.model_dump(),
        )

    def model_dump_yml(self):
        d = self.model_dump(
            mode="json",
            by_alias=True,
            exclude_none=True,
        )
        return yaml.safe_dump(d, encoding="utf-8", line_break="\n")

    basedir: Optional[RelativeDirectoryPath] = None
    action_name: Optional[str] = None
    branch_name: Optional[str] = None

    exclude: Optional[list[str]] = None
    description: str = (
        "<small>This documentation is automatically generated by "
        '<a href="https://github.com/competitive-verifier/competitive-verifier">competitive-verifier/competitive-verifier</a>'
        "</small>"
    )

    plugins: list[str] = Field(
        default_factory=lambda: [
            "jemoji",
            "jekyll-redirect-from",
            "jekyll-remote-theme",
        ]
    )

    theme: str = "jekyll-theme-minimal"
    mathjax: Union[str, int] = 3
    highlightjs_style: str = Field(
        default="default",
        serialization_alias="highlightjs-style",
    )
    filename_index: bool = Field(
        default=False,
        serialization_alias="filename-index",
    )
    sass: dict[str, Any] = Field(default_factory=lambda: {"style": "compressed"})
    icons: ConfigIcons = ConfigIcons()
    consolidate: Optional[SortedPathSet] = None


def _load_user_render_config_yml(docs_dir: pathlib.Path) -> Optional[ConfigYaml]:
    env_config_yml = os.getenv(COMPETITIVE_VERIFY_DOCS_CONFIG_YML)
    if env_config_yml:
        logger.info("Parse $COMPETITIVE_VERIFY_DOCS_CONFIG_YML: %s", env_config_yml)
        try:
            user_config_yml = yaml.safe_load(env_config_yml)
            return ConfigYaml.model_validate(user_config_yml)
        except Exception as e:
            logger.exception(
                "Failed to parse $COMPETITIVE_VERIFY_DOCS_CONFIG_YML: %s",
                e,
            )

    user_config_yml_path = docs_dir / _CONFIG_YML_PATH
    if user_config_yml_path.exists():
        logger.info("Parse user config: %s", user_config_yml_path)
        try:
            user_config_yml = yaml.safe_load(user_config_yml_path.read_bytes())
            return ConfigYaml.model_validate(user_config_yml)
        except Exception as e:
            logger.exception(
                "Failed to parse %s: %s", user_config_yml_path.as_posix(), e
            )


def load_config_yml(docs_dir: pathlib.Path) -> ConfigYaml:
    logger.info("docs_dir=%s", docs_dir.as_posix())

    config_yml = _load_user_render_config_yml(docs_dir)
    if not config_yml:
        config_yml = ConfigYaml()

    if not config_yml.action_name:
        config_yml.action_name = github.env.get_workflow_name()
    if not config_yml.branch_name:
        config_yml.branch_name = github.env.get_ref_name()

    if not config_yml.basedir:
        git_root = git.get_root_directory().resolve()
        config_yml.basedir = pathlib.Path.cwd().resolve().relative_to(git_root)
    return config_yml



import json

from pydantic import BaseModel

import competitive_verifier.models


def dump_json_schema(cls: type[BaseModel]):
    return json.dumps(
        cls.model_json_schema(),
        indent=2,
    )


def show_verify_json_schema():
    print(dump_json_schema(competitive_verifier.models.VerificationInput))


def show_result_json_schema():
    print(dump_json_schema(competitive_verifier.models.VerifyCommandResult))
